# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject
from typing import ClassVar, Optional
from typing_extensions import Literal


class CustomerPaymentToken(StripeObject):
    """
    A read-only representation of a user's PaymentMethod for use in Crypto On Ramp transactions.
    """

    OBJECT_NAME: ClassVar[Literal["crypto.payment_token"]] = (
        "crypto.payment_token"
    )

    class Card(StripeObject):
        class Wallet(StripeObject):
            type: Literal["apple_pay", "google_pay"]
            """
            The type of the card wallet, one of `apple_pay` or `google_pay`.
            """

        brand: Optional[str]
        """
        Card brand. Can be `amex`, `cartes_bancaires`, `diners`, `discover`, `eftpos_au`, `jcb`, `link`, `mastercard`, `unionpay`, `visa` or `unknown`.
        """
        exp_month: Optional[int]
        """
        Two-digit number representing the card's expiration month.
        """
        exp_year: Optional[int]
        """
        Four-digit number representing the card's expiration year.
        """
        funding: str
        """
        Card funding type. Can be `credit`, `debit`, `prepaid`, or `unknown`.
        """
        last4: Optional[str]
        """
        The last four digits of the card.
        """
        wallet: Optional[Wallet]
        """
        If this Card is part of a card wallet, this contains the details of the card wallet.
        """
        _inner_class_types = {"wallet": Wallet}

    class UsBankAccount(StripeObject):
        account_type: Optional[Literal["checking", "savings"]]
        """
        Account type: `checkings` or `savings`.
        """
        bank_name: Optional[str]
        """
        The name of the bank.
        """
        last4: Optional[str]
        """
        Last four digits of the bank account number.
        """

    card: Optional[Card]
    """
    A `card` PaymentToken, this hash contains details of the card PaymentToken.
    """
    id: str
    """
    Unique identifier for the object.
    """
    object: Literal["crypto.payment_token"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    type: Literal["card", "us_bank_account"]
    """
    Type of the Payment Token.
    """
    us_bank_account: Optional[UsBankAccount]
    """
    A `us_bank_account` PaymentToken, this hash contains details of the US bank account PaymentToken.
    """
    _inner_class_types = {"card": Card, "us_bank_account": UsBankAccount}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing_extensions import NotRequired, TypedDict


class SpendModifierRuleListParams(TypedDict):
    effective_at: NotRequired[str]
    """
    Return only spend modifiers that are effective at the specified timestamp.
    """
    limit: NotRequired[int]
    """
    Optionally set the maximum number of results per page. Defaults to 20.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, List, Optional
from typing_extensions import NotRequired, TypedDict


class FinancialAccountUpdateParams(TypedDict):
    display_name: NotRequired[str]
    """
    A descriptive name for the FinancialAccount, up to 50 characters long. This name will be used in the Stripe Dashboard and embedded components.
    """
    metadata: NotRequired[
        "Dict[str, Optional[str]]|UntypedStripeObject[Optional[str]]"
    ]
    """
    Metadata associated with the FinancialAccount.
    """
    storage: NotRequired["FinancialAccountUpdateParamsStorage"]
    """
    Parameters for updating storage-specific fields on the FinancialAccount.
    """


class FinancialAccountUpdateParamsStorage(TypedDict):
    holds_currencies: NotRequired[List[str]]
    """
    The currencies that this storage FinancialAccount can hold a balance in. Three-letter ISO currency code, in lowercase.
    Adding currencies requires the corresponding holds_currencies storer capabilities to be enabled.
    Removing currencies is not supported as of March 2026.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from decimal import Decimal
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, List
from typing_extensions import Literal, NotRequired, TypedDict


class IntentCreateParams(TypedDict):
    actions: List["IntentCreateParamsAction"]
    """
    Actions to be performed by this Billing Intent.
    """
    cadence: NotRequired[str]
    """
    ID of an existing Cadence to use.
    """
    cadence_data: NotRequired["IntentCreateParamsCadenceData"]
    """
    Data for creating a new Cadence.
    """
    currency: str
    """
    Three-letter ISO currency code, in lowercase. Must be a supported currency.
    """


class IntentCreateParamsAction(TypedDict):
    apply: NotRequired["IntentCreateParamsActionApply"]
    """
    Details for an apply action.
    """
    deactivate: NotRequired["IntentCreateParamsActionDeactivate"]
    """
    Details for a deactivate action.
    """
    modify: NotRequired["IntentCreateParamsActionModify"]
    """
    Details for a modify action.
    """
    remove: NotRequired["IntentCreateParamsActionRemove"]
    """
    Details for a remove action.
    """
    subscribe: NotRequired["IntentCreateParamsActionSubscribe"]
    """
    Details for a subscribe action.
    """
    type: Literal["apply", "deactivate", "modify", "remove", "subscribe"]
    """
    Type of the Billing Intent action.
    """


class IntentCreateParamsActionApply(TypedDict):
    discount: NotRequired["IntentCreateParamsActionApplyDiscount"]
    """
    Details for applying a discount.
    """
    effective_at: NotRequired["IntentCreateParamsActionApplyEffectiveAt"]
    """
    When the apply action takes effect. If not specified, defaults to on_reserve.
    """
    invoice_discount_rule: NotRequired[
        "IntentCreateParamsActionApplyInvoiceDiscountRule"
    ]
    """
    Details for applying a discount rule to future invoices.
    """
    spend_modifier_rule: NotRequired[
        "IntentCreateParamsActionApplySpendModifierRule"
    ]
    """
    Details for applying a spend modifier rule. Only present if type is spend_modifier_rule.
    """
    type: Literal["discount", "invoice_discount_rule", "spend_modifier_rule"]
    """
    Type of the apply action details.
    """


class IntentCreateParamsActionApplyDiscount(TypedDict):
    coupon: NotRequired[str]
    """
    The ID of the Coupon to apply.
    """
    promotion_code: NotRequired[str]
    """
    The ID of the PromotionCode to apply.
    """
    type: Literal["coupon", "promotion_code"]
    """
    Type of the discount.
    """


class IntentCreateParamsActionApplyEffectiveAt(TypedDict):
    timestamp: NotRequired[str]
    """
    The timestamp at which the apply action takes effect. Only present if type is timestamp. Only allowed for discount actions.
    """
    type: Literal[
        "current_billing_period_end",
        "current_billing_period_start",
        "next_billing_period_start",
        "on_reserve",
        "timestamp",
    ]
    """
    When the apply action takes effect.
    """


class IntentCreateParamsActionApplyInvoiceDiscountRule(TypedDict):
    applies_to: Literal["cadence"]
    """
    The entity that the discount rule applies to, for example, the cadence.
    """
    percent_off: NotRequired[
        "IntentCreateParamsActionApplyInvoiceDiscountRulePercentOff"
    ]
    """
    Configuration for percentage off discount.
    """
    type: Literal["percent_off"]
    """
    Type of the discount rule.
    """


class IntentCreateParamsActionApplyInvoiceDiscountRulePercentOff(TypedDict):
    maximum_applications: "IntentCreateParamsActionApplyInvoiceDiscountRulePercentOffMaximumApplications"
    """
    The maximum number of times this discount can be applied for this cadence.
    """
    percent_off: Decimal
    """
    Percent that is taken off the amount. For example, a percent_off of 50.0 reduces a 100 USD amount to 50 USD.
    """


class IntentCreateParamsActionApplyInvoiceDiscountRulePercentOffMaximumApplications(
    TypedDict,
):
    type: Literal["indefinite"]
    """
    The type of maximum applications configuration.
    """


class IntentCreateParamsActionApplySpendModifierRule(TypedDict):
    applies_to: Literal["cadence"]
    """
    What the spend modifier applies to.
    """
    max_billing_period_spend: NotRequired[
        "IntentCreateParamsActionApplySpendModifierRuleMaxBillingPeriodSpend"
    ]
    """
    Details for max billing period spend modifier. Only present if type is max_billing_period_spend.
    """
    type: Literal["max_billing_period_spend"]
    """
    Type of the spend modifier.
    """


class IntentCreateParamsActionApplySpendModifierRuleMaxBillingPeriodSpend(
    TypedDict,
):
    amount: "IntentCreateParamsActionApplySpendModifierRuleMaxBillingPeriodSpendAmount"
    """
    The maximum amount allowed for the billing period.
    """
    custom_pricing_unit_overage_rate: "IntentCreateParamsActionApplySpendModifierRuleMaxBillingPeriodSpendCustomPricingUnitOverageRate"
    """
    The configuration for the overage rate for the custom pricing unit.
    """


class IntentCreateParamsActionApplySpendModifierRuleMaxBillingPeriodSpendAmount(
    TypedDict,
):
    custom_pricing_unit: NotRequired[
        "IntentCreateParamsActionApplySpendModifierRuleMaxBillingPeriodSpendAmountCustomPricingUnit"
    ]
    """
    The custom pricing unit amount.
    """
    type: Literal["custom_pricing_unit"]
    """
    The type of the amount.
    """


class IntentCreateParamsActionApplySpendModifierRuleMaxBillingPeriodSpendAmountCustomPricingUnit(
    TypedDict,
):
    id: NotRequired[str]
    """
    The id of the custom pricing unit.
    """
    value: str
    """
    The value of the custom pricing unit.
    """


class IntentCreateParamsActionApplySpendModifierRuleMaxBillingPeriodSpendCustomPricingUnitOverageRate(
    TypedDict,
):
    id: str
    """
    ID of the custom pricing unit overage rate.
    """


class IntentCreateParamsActionDeactivate(TypedDict):
    cancellation_details: NotRequired[
        "IntentCreateParamsActionDeactivateCancellationDetails"
    ]
    """
    Details about why the cancellation is being requested.
    """
    collect_at: NotRequired[Literal["next_billing_date", "on_effective_at"]]
    """
    When the invoice is collected. If not specified, the default behavior is on_effective_at.
    """
    effective_at: NotRequired["IntentCreateParamsActionDeactivateEffectiveAt"]
    """
    When the deactivate action takes effect. If not specified, the default behavior is on_reserve.
    """
    pricing_plan_subscription_details: (
        "IntentCreateParamsActionDeactivatePricingPlanSubscriptionDetails"
    )
    """
    Details for deactivating a pricing plan subscription.
    """
    type: Literal[
        "pricing_plan_subscription_details", "v1_subscription_details"
    ]
    """
    Type of the action details.
    """


class IntentCreateParamsActionDeactivateCancellationDetails(TypedDict):
    comment: NotRequired[str]
    """
    Additional comments about why the user canceled the subscription, if the subscription was canceled explicitly by the user.
    """
    feedback: NotRequired[
        Literal[
            "customer_service",
            "low_quality",
            "missing_features",
            "other",
            "switched_service",
            "too_complex",
            "too_expensive",
            "unused",
        ]
    ]
    """
    The customer submitted reason for why they canceled, if the subscription was canceled explicitly by the user.
    """


class IntentCreateParamsActionDeactivateEffectiveAt(TypedDict):
    timestamp: NotRequired[str]
    """
    The timestamp at which the deactivate action takes effect. Only present if type is timestamp.
    """
    type: Literal[
        "current_billing_period_end",
        "current_billing_period_start",
        "on_reserve",
        "timestamp",
    ]
    """
    When the deactivate action takes effect.
    """


class IntentCreateParamsActionDeactivatePricingPlanSubscriptionDetails(
    TypedDict,
):
    overrides: NotRequired[
        "IntentCreateParamsActionDeactivatePricingPlanSubscriptionDetailsOverrides"
    ]
    """
    Configurations for overriding behaviors related to the subscription.
    """
    pricing_plan_subscription: str
    """
    ID of the pricing plan subscription to deactivate.
    """


class IntentCreateParamsActionDeactivatePricingPlanSubscriptionDetailsOverrides(
    TypedDict,
):
    partial_period_behaviors: List[
        "IntentCreateParamsActionDeactivatePricingPlanSubscriptionDetailsOverridesPartialPeriodBehavior"
    ]
    """
    Configurations for behaviors when the action takes effect during the service period.
    """


class IntentCreateParamsActionDeactivatePricingPlanSubscriptionDetailsOverridesPartialPeriodBehavior(
    TypedDict,
):
    license_fee: NotRequired[
        "IntentCreateParamsActionDeactivatePricingPlanSubscriptionDetailsOverridesPartialPeriodBehaviorLicenseFee"
    ]
    """
    Overrides the behavior for license fee components when the action takes effect during the service period.
    """
    type: Literal["license_fee", "recurring_credit_grant"]
    """
    The type of behavior to override.
    """


class IntentCreateParamsActionDeactivatePricingPlanSubscriptionDetailsOverridesPartialPeriodBehaviorLicenseFee(
    TypedDict,
):
    credit_proration_behavior: Literal["none", "prorated"]
    """
    The proration behavior for the partial servicing period. Defines how we prorate the license fee when the user is deactivating. If not specified, defaults to none.
    """


class IntentCreateParamsActionModify(TypedDict):
    collect_at: NotRequired[Literal["next_billing_date", "on_effective_at"]]
    """
    When the invoice is collected. If not specified, the default behavior is next_billing_date.
    """
    effective_at: NotRequired["IntentCreateParamsActionModifyEffectiveAt"]
    """
    When the modify action takes effect. If not specified, the default behavior is on_reserve.
    """
    pricing_plan_subscription_details: (
        "IntentCreateParamsActionModifyPricingPlanSubscriptionDetails"
    )
    """
    Details for modifying a pricing plan subscription.
    """
    type: Literal[
        "pricing_plan_subscription_details", "v1_subscription_details"
    ]
    """
    Type of the action details.
    """


class IntentCreateParamsActionModifyEffectiveAt(TypedDict):
    timestamp: NotRequired[str]
    """
    The timestamp at which the modify action takes effect. Only present if type is timestamp.
    """
    type: Literal["current_billing_period_start", "on_reserve", "timestamp"]
    """
    When the modify action takes effect.
    """


class IntentCreateParamsActionModifyPricingPlanSubscriptionDetails(TypedDict):
    component_configurations: NotRequired[
        List[
            "IntentCreateParamsActionModifyPricingPlanSubscriptionDetailsComponentConfiguration"
        ]
    ]
    """
    New configurations for the components of the pricing plan.
    """
    new_pricing_plan: NotRequired[str]
    """
    The ID of the new Pricing Plan, if changing plans.
    """
    new_pricing_plan_version: NotRequired[str]
    """
    The ID of the new Pricing Plan Version to use.
    """
    overrides: NotRequired[
        "IntentCreateParamsActionModifyPricingPlanSubscriptionDetailsOverrides"
    ]
    """
    Configurations for overriding behaviors related to the subscription.
    """
    pricing_plan_subscription: str
    """
    The ID of the Pricing Plan Subscription to modify.
    """


class IntentCreateParamsActionModifyPricingPlanSubscriptionDetailsComponentConfiguration(
    TypedDict,
):
    lookup_key: NotRequired[str]
    """
    Lookup key for the pricing plan component.
    """
    pricing_plan_component: NotRequired[str]
    """
    ID of the pricing plan component.
    """
    quantity: NotRequired[int]
    """
    Quantity of the component to be used.
    """


class IntentCreateParamsActionModifyPricingPlanSubscriptionDetailsOverrides(
    TypedDict,
):
    partial_period_behaviors: List[
        "IntentCreateParamsActionModifyPricingPlanSubscriptionDetailsOverridesPartialPeriodBehavior"
    ]
    """
    Configurations for behaviors when the action takes effect during the service period.
    """


class IntentCreateParamsActionModifyPricingPlanSubscriptionDetailsOverridesPartialPeriodBehavior(
    TypedDict,
):
    license_fee: NotRequired[
        "IntentCreateParamsActionModifyPricingPlanSubscriptionDetailsOverridesPartialPeriodBehaviorLicenseFee"
    ]
    """
    Overrides the behavior for license fee components when the action takes effect during the service period.
    """
    recurring_credit_grant: NotRequired[
        "IntentCreateParamsActionModifyPricingPlanSubscriptionDetailsOverridesPartialPeriodBehaviorRecurringCreditGrant"
    ]
    """
    Overrides the behavior for recurring credit grant components when the action takes effect during the service period.
    """
    type: Literal["license_fee", "recurring_credit_grant"]
    """
    The type of behavior to override.
    """


class IntentCreateParamsActionModifyPricingPlanSubscriptionDetailsOverridesPartialPeriodBehaviorLicenseFee(
    TypedDict,
):
    credit_proration_behavior: Literal["none", "prorated"]
    """
    The proration behavior for the partial servicing period. Defines how we prorate the license fee when the user modifies the subscription. If not specified, defaults to prorated.
    """
    debit_proration_behavior: Literal["none", "prorated"]
    """
    The proration behavior for the partial servicing period. Defines how we prorate the license fee when the user modifies the subscription. If not specified, defaults to prorated.
    """


class IntentCreateParamsActionModifyPricingPlanSubscriptionDetailsOverridesPartialPeriodBehaviorRecurringCreditGrant(
    TypedDict,
):
    create_behavior: Literal["full_credits", "none"]
    """
    Controls credit grant creation behavior during partial periods. If not specified, defaults to full_credits.
    """


class IntentCreateParamsActionRemove(TypedDict):
    effective_at: NotRequired["IntentCreateParamsActionRemoveEffectiveAt"]
    """
    When the remove action takes effect. If not specified, defaults to on_reserve.
    """
    invoice_discount_rule: NotRequired[str]
    """
    The ID of the discount rule to remove for future invoices.
    """
    spend_modifier_rule: NotRequired[str]
    """
    The ID of the spend modifier rule to remove.
    """
    type: Literal["invoice_discount_rule", "spend_modifier_rule"]
    """
    Type of the remove action.
    """


class IntentCreateParamsActionRemoveEffectiveAt(TypedDict):
    type: Literal["current_billing_period_end", "on_reserve"]
    """
    When the remove action takes effect.
    """


class IntentCreateParamsActionSubscribe(TypedDict):
    collect_at: NotRequired[Literal["next_billing_date", "on_effective_at"]]
    """
    When the invoice is collected. If not specified, defaults to on_effective_at.
    """
    effective_at: NotRequired["IntentCreateParamsActionSubscribeEffectiveAt"]
    """
    When the subscribe action takes effect. If not specified, the default behavior is on_reserve.
    """
    pricing_plan_subscription_details: NotRequired[
        "IntentCreateParamsActionSubscribePricingPlanSubscriptionDetails"
    ]
    """
    Details for subscribing to a pricing plan.
    """
    type: Literal[
        "pricing_plan_subscription_details", "v1_subscription_details"
    ]
    """
    Type of the action details.
    """
    v1_subscription_details: NotRequired[
        "IntentCreateParamsActionSubscribeV1SubscriptionDetails"
    ]
    """
    Details for subscribing to a v1 subscription.
    """


class IntentCreateParamsActionSubscribeEffectiveAt(TypedDict):
    timestamp: NotRequired[str]
    """
    The timestamp at which the subscribe action takes effect. Only present if type is timestamp.
    """
    type: Literal["current_billing_period_start", "on_reserve", "timestamp"]
    """
    When the subscribe action takes effect.
    """


class IntentCreateParamsActionSubscribePricingPlanSubscriptionDetails(
    TypedDict,
):
    component_configurations: NotRequired[
        List[
            "IntentCreateParamsActionSubscribePricingPlanSubscriptionDetailsComponentConfiguration"
        ]
    ]
    """
    Configurations for the components of the pricing plan.
    """
    metadata: NotRequired["Dict[str, str]|UntypedStripeObject[str]"]
    """
    Set of [key-value pairs](https://docs.stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    overrides: NotRequired[
        "IntentCreateParamsActionSubscribePricingPlanSubscriptionDetailsOverrides"
    ]
    """
    Configurations for overriding behaviors related to the subscription.
    """
    pricing_plan: str
    """
    ID of the Pricing Plan to subscribe to.
    """
    pricing_plan_version: str
    """
    Version of the Pricing Plan to use.
    """


class IntentCreateParamsActionSubscribePricingPlanSubscriptionDetailsComponentConfiguration(
    TypedDict,
):
    lookup_key: NotRequired[str]
    """
    Lookup key for the pricing plan component.
    """
    pricing_plan_component: NotRequired[str]
    """
    ID of the pricing plan component.
    """
    quantity: NotRequired[int]
    """
    Quantity of the component to be used.
    """


class IntentCreateParamsActionSubscribePricingPlanSubscriptionDetailsOverrides(
    TypedDict,
):
    partial_period_behaviors: List[
        "IntentCreateParamsActionSubscribePricingPlanSubscriptionDetailsOverridesPartialPeriodBehavior"
    ]
    """
    Configurations for behaviors when the action takes effect during the service period.
    """


class IntentCreateParamsActionSubscribePricingPlanSubscriptionDetailsOverridesPartialPeriodBehavior(
    TypedDict,
):
    license_fee: NotRequired[
        "IntentCreateParamsActionSubscribePricingPlanSubscriptionDetailsOverridesPartialPeriodBehaviorLicenseFee"
    ]
    """
    Overrides the behavior for license fee components when the action takes effect during the service period.
    """
    recurring_credit_grant: NotRequired[
        "IntentCreateParamsActionSubscribePricingPlanSubscriptionDetailsOverridesPartialPeriodBehaviorRecurringCreditGrant"
    ]
    """
    Overrides the behavior for recurring credit grant components when the action takes effect during the service period.
    """
    type: Literal["license_fee", "recurring_credit_grant"]
    """
    The type of behavior to override.
    """


class IntentCreateParamsActionSubscribePricingPlanSubscriptionDetailsOverridesPartialPeriodBehaviorLicenseFee(
    TypedDict,
):
    debit_proration_behavior: Literal["none", "prorated"]
    """
    The proration behavior for the partial servicing period. Defines how we prorate the license fee when the user is subscribing. If not specified, defaults to prorated.
    """


class IntentCreateParamsActionSubscribePricingPlanSubscriptionDetailsOverridesPartialPeriodBehaviorRecurringCreditGrant(
    TypedDict,
):
    create_behavior: Literal["full_credits", "none"]
    """
    Controls credit grant creation behavior during partial periods. If not specified, defaults to full_credits.
    """


class IntentCreateParamsActionSubscribeV1SubscriptionDetails(TypedDict):
    description: NotRequired[str]
    """
    The subscription's description, meant to be displayable to the customer.
    Use this field to optionally store an explanation of the subscription for rendering in Stripe surfaces and certain local payment methods UIs.
    """
    items: List["IntentCreateParamsActionSubscribeV1SubscriptionDetailsItem"]
    """
    A list of up to 20 subscription items, each with an attached price.
    """
    metadata: NotRequired["Dict[str, str]|UntypedStripeObject[str]"]
    """
    Set of key-value pairs that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """


class IntentCreateParamsActionSubscribeV1SubscriptionDetailsItem(TypedDict):
    metadata: NotRequired["Dict[str, str]|UntypedStripeObject[str]"]
    """
    Set of key-value pairs that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    price: str
    """
    The ID of the price object.
    """
    quantity: NotRequired[int]
    """
    Quantity for this item. If not provided, defaults to 1.
    """


class IntentCreateParamsCadenceData(TypedDict):
    billing_cycle: "IntentCreateParamsCadenceDataBillingCycle"
    """
    The billing cycle configuration for this Cadence.
    """
    payer: "IntentCreateParamsCadenceDataPayer"
    """
    Information about the payer for this Cadence.
    """
    settings: NotRequired["IntentCreateParamsCadenceDataSettings"]
    """
    Settings for creating the Cadence.
    """


class IntentCreateParamsCadenceDataBillingCycle(TypedDict):
    day: NotRequired["IntentCreateParamsCadenceDataBillingCycleDay"]
    """
    Specific configuration for determining billing dates when type=day.
    """
    interval_count: NotRequired[int]
    """
    The number of intervals (specified in the interval attribute) between
    cadence billings. For example, type=month and interval_count=3 bills every
    3 months. If not provided, this defaults to 1.
    """
    month: NotRequired["IntentCreateParamsCadenceDataBillingCycleMonth"]
    """
    Specific configuration for determining billing dates when type=month.
    """
    type: Literal["day", "month", "week", "year"]
    """
    The frequency at which a cadence bills.
    """
    week: NotRequired["IntentCreateParamsCadenceDataBillingCycleWeek"]
    """
    Specific configuration for determining billing dates when type=week.
    """
    year: NotRequired["IntentCreateParamsCadenceDataBillingCycleYear"]
    """
    Specific configuration for determining billing dates when type=year.
    """


class IntentCreateParamsCadenceDataBillingCycleDay(TypedDict):
    time: NotRequired["IntentCreateParamsCadenceDataBillingCycleDayTime"]
    """
    The time at which the billing cycle ends.
    This field is optional, and if not provided, it defaults to
    the time at which the cadence was created in UTC timezone.
    """


class IntentCreateParamsCadenceDataBillingCycleDayTime(TypedDict):
    hour: int
    """
    The hour at which the billing cycle ends.
    This must be an integer between 0 and 23, inclusive.
    0 represents midnight, and 23 represents 11 PM.
    """
    minute: int
    """
    The minute at which the billing cycle ends.
    Must be an integer between 0 and 59, inclusive.
    """
    second: int
    """
    The second at which the billing cycle ends.
    Must be an integer between 0 and 59, inclusive.
    """


class IntentCreateParamsCadenceDataBillingCycleMonth(TypedDict):
    day_of_month: int
    """
    The day to anchor the billing on for a type="month" billing cycle from
    1-31. If this number is greater than the number of days in the month being
    billed, this anchors to the last day of the month. If not provided,
    this defaults to the day the cadence was created.
    """
    month_of_year: NotRequired[int]
    """
    The month to anchor the billing on for a type="month" billing cycle from
    1-12. If not provided, this defaults to the month the cadence was created.
    This setting can only be used for monthly billing cycles with `interval_count` of 2, 3, 4 or 6.
    All occurrences are calculated from the month provided.
    """
    time: NotRequired["IntentCreateParamsCadenceDataBillingCycleMonthTime"]
    """
    The time at which the billing cycle ends.
    This field is optional, and if not provided, it defaults to
    the time at which the cadence was created in UTC timezone.
    """


class IntentCreateParamsCadenceDataBillingCycleMonthTime(TypedDict):
    hour: int
    """
    The hour at which the billing cycle ends.
    This must be an integer between 0 and 23, inclusive.
    0 represents midnight, and 23 represents 11 PM.
    """
    minute: int
    """
    The minute at which the billing cycle ends.
    Must be an integer between 0 and 59, inclusive.
    """
    second: int
    """
    The second at which the billing cycle ends.
    Must be an integer between 0 and 59, inclusive.
    """


class IntentCreateParamsCadenceDataBillingCycleWeek(TypedDict):
    day_of_week: int
    """
    The day of the week to bill the type=week billing cycle on.
    Numbered from 1-7 for Monday to Sunday respectively, based on the ISO-8601
    week day numbering. If not provided, this defaults to the day the
    cadence was created.
    """
    time: NotRequired["IntentCreateParamsCadenceDataBillingCycleWeekTime"]
    """
    The time at which the billing cycle ends.
    This field is optional, and if not provided, it defaults to
    the time at which the cadence was created in UTC timezone.
    """


class IntentCreateParamsCadenceDataBillingCycleWeekTime(TypedDict):
    hour: int
    """
    The hour at which the billing cycle ends.
    This must be an integer between 0 and 23, inclusive.
    0 represents midnight, and 23 represents 11 PM.
    """
    minute: int
    """
    The minute at which the billing cycle ends.
    Must be an integer between 0 and 59, inclusive.
    """
    second: int
    """
    The second at which the billing cycle ends.
    Must be an integer between 0 and 59, inclusive.
    """


class IntentCreateParamsCadenceDataBillingCycleYear(TypedDict):
    day_of_month: NotRequired[int]
    """
    The day to anchor the billing on for a type="month" billing cycle from
    1-31. If this number is greater than the number of days in the month being
    billed, this anchors to the last day of the month. If not provided,
    this defaults to the day the cadence was created.
    """
    month_of_year: NotRequired[int]
    """
    The month to bill on from 1-12. If not provided, this defaults to the
    month the cadence was created.
    """
    time: NotRequired["IntentCreateParamsCadenceDataBillingCycleYearTime"]
    """
    The time at which the billing cycle ends.
    This field is optional, and if not provided, it defaults to
    the time at which the cadence was created in UTC timezone.
    """


class IntentCreateParamsCadenceDataBillingCycleYearTime(TypedDict):
    hour: int
    """
    The hour at which the billing cycle ends.
    This must be an integer between 0 and 23, inclusive.
    0 represents midnight, and 23 represents 11 PM.
    """
    minute: int
    """
    The minute at which the billing cycle ends.
    Must be an integer between 0 and 59, inclusive.
    """
    second: int
    """
    The second at which the billing cycle ends.
    Must be an integer between 0 and 59, inclusive.
    """


class IntentCreateParamsCadenceDataPayer(TypedDict):
    billing_profile: NotRequired[str]
    """
    The ID of the Billing Profile object which determines how a bill is paid.
    """
    billing_profile_data: NotRequired[
        "IntentCreateParamsCadenceDataPayerBillingProfileData"
    ]
    """
    Data for creating a new profile.
    """


class IntentCreateParamsCadenceDataPayerBillingProfileData(TypedDict):
    customer: str
    """
    The customer to associate with the profile.
    """
    default_payment_method: NotRequired[str]
    """
    The default payment method to use when billing this profile.
    If left blank, the `PaymentMethod` from the `PaymentIntent` provided
    on commit is used to create the profile.
    """


class IntentCreateParamsCadenceDataSettings(TypedDict):
    bill: NotRequired["IntentCreateParamsCadenceDataSettingsBill"]
    """
    Settings that configure bill generation, which includes calculating totals, tax, and presenting invoices.
    If no setting is provided here, the settings from the customer referenced on the payer will be used.
    If no customer settings are present, the merchant default settings will be used.
    """
    collection: NotRequired["IntentCreateParamsCadenceDataSettingsCollection"]
    """
    Settings that configure and manage the behavior of collecting payments.
    If no setting is provided here, the settings from the customer referenced from the payer will be used if they exist.
    If no customer settings are present, the merchant default settings will be used.
    """


class IntentCreateParamsCadenceDataSettingsBill(TypedDict):
    id: str
    """
    The ID of the referenced settings object.
    """
    version: NotRequired[str]
    """
    An optional field to specify the version of the Settings to use.
    If not provided, this defaults to the live version any time the settings are used.
    """


class IntentCreateParamsCadenceDataSettingsCollection(TypedDict):
    id: str
    """
    The ID of the referenced settings object.
    """
    version: NotRequired[str]
    """
    An optional field to specify the version of the Settings to use.
    If not provided, this defaults to the live version any time the settings are used.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import List
from typing_extensions import NotRequired


class ReleaseListParams(RequestOptions):
    currency: NotRequired[str]
    """
    Only return ReserveReleases associated with the currency specified by this currency code. Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    ending_before: NotRequired[str]
    """
    A cursor for use in pagination. `ending_before` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, starting with `obj_bar`, your subsequent call can include `ending_before=obj_bar` in order to fetch the previous page of the list.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    limit: NotRequired[int]
    """
    A limit on the number of objects to be returned. Limit can range between 1 and 100, and the default is 10.
    """
    reserve_hold: NotRequired[str]
    """
    Only return ReserveReleases associated with the ReserveHold specified by this ReserveHold ID.
    """
    reserve_plan: NotRequired[str]
    """
    Only return ReserveReleases associated with the ReservePlan specified by this ReservePlan ID.
    """
    starting_after: NotRequired[str]
    """
    A cursor for use in pagination. `starting_after` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with `obj_foo`, your subsequent call can include `starting_after=obj_foo` in order to fetch the next page of the list.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, List
from typing_extensions import Literal, NotRequired, TypedDict


class CreditUnderwritingRecordCreateFromApplicationParams(RequestOptions):
    application: (
        "CreditUnderwritingRecordCreateFromApplicationParamsApplication"
    )
    """
    Details about the application submission.
    """
    credit_user: (
        "CreditUnderwritingRecordCreateFromApplicationParamsCreditUser"
    )
    """
    Information about the company or person applying or holding the account.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    metadata: NotRequired["Dict[str, str]|UntypedStripeObject[str]"]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """


class CreditUnderwritingRecordCreateFromApplicationParamsApplication(
    TypedDict
):
    application_method: NotRequired[
        Literal["in_person", "mail", "online", "phone"]
    ]
    """
    The channel through which the applicant has submitted their application. Defaults to `online`.
    """
    purpose: Literal["credit_limit_increase", "credit_line_opening"]
    """
    Scope of demand made by the applicant.
    """
    submitted_at: int
    """
    Date when the applicant submitted their application.
    """


class CreditUnderwritingRecordCreateFromApplicationParamsCreditUser(TypedDict):
    email: str
    """
    Email of the applicant or accountholder.
    """
    name: str
    """
    Full name of the company or person.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing import List
from typing_extensions import Literal, NotRequired, TypedDict


class ActivityLogListParams(TypedDict):
    action_groups: NotRequired[
        List[Literal["api_key", "user_invite", "user_roles"]]
    ]
    """
    Filter results to only include activity logs for the specified action group types.
    """
    actions: NotRequired[
        List[
            Literal[
                "api_key_created",
                "api_key_deleted",
                "api_key_updated",
                "api_key_viewed",
                "user_invite_accepted",
                "user_invite_created",
                "user_invite_deleted",
                "user_roles_deleted",
                "user_roles_updated",
            ]
        ]
    ]
    """
    Filter results to only include activity logs for the specified action types.
    """
    limit: NotRequired[int]
    """
    Maximum number of results to return per page.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.params.v2.core.vault._gb_bank_account_acknowledge_confirmation_of_payee_params import (
        GbBankAccountAcknowledgeConfirmationOfPayeeParams,
    )
    from stripe.params.v2.core.vault._gb_bank_account_archive_params import (
        GbBankAccountArchiveParams,
    )
    from stripe.params.v2.core.vault._gb_bank_account_create_params import (
        GbBankAccountCreateParams,
    )
    from stripe.params.v2.core.vault._gb_bank_account_initiate_confirmation_of_payee_params import (
        GbBankAccountInitiateConfirmationOfPayeeParams,
    )
    from stripe.params.v2.core.vault._gb_bank_account_list_params import (
        GbBankAccountListParams,
    )
    from stripe.params.v2.core.vault._gb_bank_account_retrieve_params import (
        GbBankAccountRetrieveParams,
    )
    from stripe.v2._list_object import ListObject
    from stripe.v2.core.vault._gb_bank_account import GbBankAccount


class GbBankAccountService(StripeService):
    def list(
        self,
        params: Optional["GbBankAccountListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[GbBankAccount]":
        """
        List objects that can be used as destinations for outbound money movement via OutboundPayment.
        """
        return cast(
            "ListObject[GbBankAccount]",
            self._request(
                "get",
                "/v2/core/vault/gb_bank_accounts",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["GbBankAccountListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[GbBankAccount]":
        """
        List objects that can be used as destinations for outbound money movement via OutboundPayment.
        """
        return cast(
            "ListObject[GbBankAccount]",
            await self._request_async(
                "get",
                "/v2/core/vault/gb_bank_accounts",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "GbBankAccountCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "GbBankAccount":
        """
        Create a GB bank account.
        """
        return cast(
            "GbBankAccount",
            self._request(
                "post",
                "/v2/core/vault/gb_bank_accounts",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "GbBankAccountCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "GbBankAccount":
        """
        Create a GB bank account.
        """
        return cast(
            "GbBankAccount",
            await self._request_async(
                "post",
                "/v2/core/vault/gb_bank_accounts",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: Optional["GbBankAccountRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "GbBankAccount":
        """
        Retrieve a GB bank account.
        """
        return cast(
            "GbBankAccount",
            self._request(
                "get",
                "/v2/core/vault/gb_bank_accounts/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["GbBankAccountRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "GbBankAccount":
        """
        Retrieve a GB bank account.
        """
        return cast(
            "GbBankAccount",
            await self._request_async(
                "get",
                "/v2/core/vault/gb_bank_accounts/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def acknowledge_confirmation_of_payee(
        self,
        id: str,
        params: Optional[
            "GbBankAccountAcknowledgeConfirmationOfPayeeParams"
        ] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "GbBankAccount":
        """
        Confirm that you have received the result of the Confirmation of Payee request, and that you are okay with
        proceeding to pay out to this bank account despite the account not matching, partially matching, or the service
        being unavailable. Once you confirm this, you will be able to send OutboundPayments, but this may lead to
        funds being sent to the wrong account, which we might not be able to recover.
        """
        return cast(
            "GbBankAccount",
            self._request(
                "post",
                "/v2/core/vault/gb_bank_accounts/{id}/acknowledge_confirmation_of_payee".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def acknowledge_confirmation_of_payee_async(
        self,
        id: str,
        params: Optional[
            "GbBankAccountAcknowledgeConfirmationOfPayeeParams"
        ] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "GbBankAccount":
        """
        Confirm that you have received the result of the Confirmation of Payee request, and that you are okay with
        proceeding to pay out to this bank account despite the account not matching, partially matching, or the service
        being unavailable. Once you confirm this, you will be able to send OutboundPayments, but this may lead to
        funds being sent to the wrong account, which we might not be able to recover.
        """
        return cast(
            "GbBankAccount",
            await self._request_async(
                "post",
                "/v2/core/vault/gb_bank_accounts/{id}/acknowledge_confirmation_of_payee".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def archive(
        self,
        id: str,
        params: Optional["GbBankAccountArchiveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "GbBankAccount":
        """
        Archive a GBBankAccount object. Archived GBBankAccount objects cannot be used as outbound destinations
        and will not appear in the outbound destination list.
        """
        return cast(
            "GbBankAccount",
            self._request(
                "post",
                "/v2/core/vault/gb_bank_accounts/{id}/archive".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def archive_async(
        self,
        id: str,
        params: Optional["GbBankAccountArchiveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "GbBankAccount":
        """
        Archive a GBBankAccount object. Archived GBBankAccount objects cannot be used as outbound destinations
        and will not appear in the outbound destination list.
        """
        return cast(
            "GbBankAccount",
            await self._request_async(
                "post",
                "/v2/core/vault/gb_bank_accounts/{id}/archive".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def initiate_confirmation_of_payee(
        self,
        id: str,
        params: Optional[
            "GbBankAccountInitiateConfirmationOfPayeeParams"
        ] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "GbBankAccount":
        """
        Initiate Confirmation of Payee (CoP) in order to verify that the owner of a UK bank account matches
        who you expect. This must be done on all UK bank accounts before sending domestic OutboundPayments. If
        the result is a partial match or a non match, explicit acknowledgement using AcknowledgeConfirmationOfPayee
        is required before sending funds.
        """
        return cast(
            "GbBankAccount",
            self._request(
                "post",
                "/v2/core/vault/gb_bank_accounts/{id}/initiate_confirmation_of_payee".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def initiate_confirmation_of_payee_async(
        self,
        id: str,
        params: Optional[
            "GbBankAccountInitiateConfirmationOfPayeeParams"
        ] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "GbBankAccount":
        """
        Initiate Confirmation of Payee (CoP) in order to verify that the owner of a UK bank account matches
        who you expect. This must be done on all UK bank accounts before sending domestic OutboundPayments. If
        the result is a partial match or a non match, explicit acknowledgement using AcknowledgeConfirmationOfPayee
        is required before sending funds.
        """
        return cast(
            "GbBankAccount",
            await self._request_async(
                "post",
                "/v2/core/vault/gb_bank_accounts/{id}/initiate_confirmation_of_payee".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, List
from typing_extensions import Literal, NotRequired, TypedDict


class AccountExternalAccountUpdateParams(TypedDict):
    account_holder_name: NotRequired[str]
    """
    The name of the person or business that owns the bank account.
    """
    account_holder_type: NotRequired[
        "Literal['']|Literal['company', 'individual']"
    ]
    """
    The type of entity that holds the account. This can be either `individual` or `company`.
    """
    account_type: NotRequired[Literal["checking", "futsu", "savings", "toza"]]
    """
    The bank account type. This can only be `checking` or `savings` in most countries. In Japan, this can only be `futsu` or `toza`.
    """
    address_city: NotRequired[str]
    """
    City/District/Suburb/Town/Village.
    """
    address_country: NotRequired[str]
    """
    Billing address country, if provided when creating card.
    """
    address_line1: NotRequired[str]
    """
    Address line 1 (Street address/PO Box/Company name).
    """
    address_line2: NotRequired[str]
    """
    Address line 2 (Apartment/Suite/Unit/Building).
    """
    address_state: NotRequired[str]
    """
    State/County/Province/Region.
    """
    address_zip: NotRequired[str]
    """
    ZIP or postal code.
    """
    default_for_currency: NotRequired[bool]
    """
    When set to true, this becomes the default external account for its currency.
    """
    documents: NotRequired["AccountExternalAccountUpdateParamsDocuments"]
    """
    Documents that may be submitted to satisfy various informational requests.
    """
    exp_month: NotRequired[str]
    """
    Two digit number representing the card's expiration month.
    """
    exp_year: NotRequired[str]
    """
    Four digit number representing the card's expiration year.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    metadata: NotRequired[
        "Literal['']|Dict[str, str]|UntypedStripeObject[str]"
    ]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    name: NotRequired[str]
    """
    Cardholder name.
    """


class AccountExternalAccountUpdateParamsDocuments(TypedDict):
    bank_account_ownership_verification: NotRequired[
        "AccountExternalAccountUpdateParamsDocumentsBankAccountOwnershipVerification"
    ]
    """
    One or more documents that support the [Bank account ownership verification](https://support.stripe.com/questions/bank-account-ownership-verification) requirement. Must be a document associated with the bank account that displays the last 4 digits of the account number, either a statement or a check.
    """


class AccountExternalAccountUpdateParamsDocumentsBankAccountOwnershipVerification(
    TypedDict,
):
    files: NotRequired[List[str]]
    """
    One or more document ids returned by a [file upload](https://api.stripe.com#create_file) with a `purpose` value of `account_requirement`.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.params.v2.data.reporting._query_run_create_params import (
        QueryRunCreateParams,
    )
    from stripe.params.v2.data.reporting._query_run_retrieve_params import (
        QueryRunRetrieveParams,
    )
    from stripe.v2.data.reporting._query_run import QueryRun


class QueryRunService(StripeService):
    def create(
        self,
        params: "QueryRunCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "QueryRun":
        """
        Creates a query run to execute ad-hoc SQL and returns a `QueryRun` object to track progress and retrieve results.
        """
        return cast(
            "QueryRun",
            self._request(
                "post",
                "/v2/data/reporting/query_runs",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "QueryRunCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "QueryRun":
        """
        Creates a query run to execute ad-hoc SQL and returns a `QueryRun` object to track progress and retrieve results.
        """
        return cast(
            "QueryRun",
            await self._request_async(
                "post",
                "/v2/data/reporting/query_runs",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: Optional["QueryRunRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "QueryRun":
        """
        Fetches the current state and details of a previously created `QueryRun`. If the `QueryRun`
        has succeeded, the endpoint will provide details for how to retrieve the results.
        """
        return cast(
            "QueryRun",
            self._request(
                "get",
                "/v2/data/reporting/query_runs/{id}".format(
                    id=sanitize_id(id)
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["QueryRunRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "QueryRun":
        """
        Fetches the current state and details of a previously created `QueryRun`. If the `QueryRun`
        has succeeded, the endpoint will provide details for how to retrieve the results.
        """
        return cast(
            "QueryRun",
            await self._request_async(
                "get",
                "/v2/data/reporting/query_runs/{id}".format(
                    id=sanitize_id(id)
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.v2.billing import (
        bill_settings as bill_settings,
        cadences as cadences,
        collection_settings as collection_settings,
        contracts as contracts,
        intents as intents,
        license_fees as license_fees,
        pricing_plans as pricing_plans,
        rate_cards as rate_cards,
    )
    from stripe.v2.billing._bill_setting import BillSetting as BillSetting
    from stripe.v2.billing._bill_setting_service import (
        BillSettingService as BillSettingService,
    )
    from stripe.v2.billing._bill_setting_version import (
        BillSettingVersion as BillSettingVersion,
    )
    from stripe.v2.billing._cadence import Cadence as Cadence
    from stripe.v2.billing._cadence_service import (
        CadenceService as CadenceService,
    )
    from stripe.v2.billing._cadence_spend_modifier import (
        CadenceSpendModifier as CadenceSpendModifier,
    )
    from stripe.v2.billing._collection_setting import (
        CollectionSetting as CollectionSetting,
    )
    from stripe.v2.billing._collection_setting_service import (
        CollectionSettingService as CollectionSettingService,
    )
    from stripe.v2.billing._collection_setting_version import (
        CollectionSettingVersion as CollectionSettingVersion,
    )
    from stripe.v2.billing._contract import Contract as Contract
    from stripe.v2.billing._contract_pricing_line_quantity_change import (
        ContractPricingLineQuantityChange as ContractPricingLineQuantityChange,
    )
    from stripe.v2.billing._contract_service import (
        ContractService as ContractService,
    )
    from stripe.v2.billing._custom_pricing_unit import (
        CustomPricingUnit as CustomPricingUnit,
    )
    from stripe.v2.billing._custom_pricing_unit_service import (
        CustomPricingUnitService as CustomPricingUnitService,
    )
    from stripe.v2.billing._intent import Intent as Intent
    from stripe.v2.billing._intent_action import IntentAction as IntentAction
    from stripe.v2.billing._intent_service import (
        IntentService as IntentService,
    )
    from stripe.v2.billing._license_fee import LicenseFee as LicenseFee
    from stripe.v2.billing._license_fee_service import (
        LicenseFeeService as LicenseFeeService,
    )
    from stripe.v2.billing._license_fee_version import (
        LicenseFeeVersion as LicenseFeeVersion,
    )
    from stripe.v2.billing._licensed_item import LicensedItem as LicensedItem
    from stripe.v2.billing._licensed_item_service import (
        LicensedItemService as LicensedItemService,
    )
    from stripe.v2.billing._meter_event import MeterEvent as MeterEvent
    from stripe.v2.billing._meter_event_adjustment import (
        MeterEventAdjustment as MeterEventAdjustment,
    )
    from stripe.v2.billing._meter_event_adjustment_service import (
        MeterEventAdjustmentService as MeterEventAdjustmentService,
    )
    from stripe.v2.billing._meter_event_service import (
        MeterEventService as MeterEventService,
    )
    from stripe.v2.billing._meter_event_session import (
        MeterEventSession as MeterEventSession,
    )
    from stripe.v2.billing._meter_event_session_service import (
        MeterEventSessionService as MeterEventSessionService,
    )
    from stripe.v2.billing._meter_event_stream_service import (
        MeterEventStreamService as MeterEventStreamService,
    )
    from stripe.v2.billing._metered_item import MeteredItem as MeteredItem
    from stripe.v2.billing._metered_item_service import (
        MeteredItemService as MeteredItemService,
    )
    from stripe.v2.billing._one_time_item import OneTimeItem as OneTimeItem
    from stripe.v2.billing._one_time_item_service import (
        OneTimeItemService as OneTimeItemService,
    )
    from stripe.v2.billing._pricing_plan import PricingPlan as PricingPlan
    from stripe.v2.billing._pricing_plan_component import (
        PricingPlanComponent as PricingPlanComponent,
    )
    from stripe.v2.billing._pricing_plan_service import (
        PricingPlanService as PricingPlanService,
    )
    from stripe.v2.billing._pricing_plan_subscription import (
        PricingPlanSubscription as PricingPlanSubscription,
    )
    from stripe.v2.billing._pricing_plan_subscription_service import (
        PricingPlanSubscriptionService as PricingPlanSubscriptionService,
    )
    from stripe.v2.billing._pricing_plan_version import (
        PricingPlanVersion as PricingPlanVersion,
    )
    from stripe.v2.billing._profile import Profile as Profile
    from stripe.v2.billing._profile_service import (
        ProfileService as ProfileService,
    )
    from stripe.v2.billing._rate_card import RateCard as RateCard
    from stripe.v2.billing._rate_card_custom_pricing_unit_overage_rate import (
        RateCardCustomPricingUnitOverageRate as RateCardCustomPricingUnitOverageRate,
    )
    from stripe.v2.billing._rate_card_rate import RateCardRate as RateCardRate
    from stripe.v2.billing._rate_card_service import (
        RateCardService as RateCardService,
    )
    from stripe.v2.billing._rate_card_subscription import (
        RateCardSubscription as RateCardSubscription,
    )
    from stripe.v2.billing._rate_card_subscription_service import (
        RateCardSubscriptionService as RateCardSubscriptionService,
    )
    from stripe.v2.billing._rate_card_version import (
        RateCardVersion as RateCardVersion,
    )
    from stripe.v2.billing._service_action import (
        ServiceAction as ServiceAction,
    )
    from stripe.v2.billing._service_action_service import (
        ServiceActionService as ServiceActionService,
    )

# name -> (import_target, is_submodule)
_import_map = {
    "bill_settings": ("stripe.v2.billing.bill_settings", True),
    "cadences": ("stripe.v2.billing.cadences", True),
    "collection_settings": ("stripe.v2.billing.collection_settings", True),
    "contracts": ("stripe.v2.billing.contracts", True),
    "intents": ("stripe.v2.billing.intents", True),
    "license_fees": ("stripe.v2.billing.license_fees", True),
    "pricing_plans": ("stripe.v2.billing.pricing_plans", True),
    "rate_cards": ("stripe.v2.billing.rate_cards", True),
    "BillSetting": ("stripe.v2.billing._bill_setting", False),
    "BillSettingService": ("stripe.v2.billing._bill_setting_service", False),
    "BillSettingVersion": ("stripe.v2.billing._bill_setting_version", False),
    "Cadence": ("stripe.v2.billing._cadence", False),
    "CadenceService": ("stripe.v2.billing._cadence_service", False),
    "CadenceSpendModifier": (
        "stripe.v2.billing._cadence_spend_modifier",
        False,
    ),
    "CollectionSetting": ("stripe.v2.billing._collection_setting", False),
    "CollectionSettingService": (
        "stripe.v2.billing._collection_setting_service",
        False,
    ),
    "CollectionSettingVersion": (
        "stripe.v2.billing._collection_setting_version",
        False,
    ),
    "Contract": ("stripe.v2.billing._contract", False),
    "ContractPricingLineQuantityChange": (
        "stripe.v2.billing._contract_pricing_line_quantity_change",
        False,
    ),
    "ContractService": ("stripe.v2.billing._contract_service", False),
    "CustomPricingUnit": ("stripe.v2.billing._custom_pricing_unit", False),
    "CustomPricingUnitService": (
        "stripe.v2.billing._custom_pricing_unit_service",
        False,
    ),
    "Intent": ("stripe.v2.billing._intent", False),
    "IntentAction": ("stripe.v2.billing._intent_action", False),
    "IntentService": ("stripe.v2.billing._intent_service", False),
    "LicenseFee": ("stripe.v2.billing._license_fee", False),
    "LicenseFeeService": ("stripe.v2.billing._license_fee_service", False),
    "LicenseFeeVersion": ("stripe.v2.billing._license_fee_version", False),
    "LicensedItem": ("stripe.v2.billing._licensed_item", False),
    "LicensedItemService": ("stripe.v2.billing._licensed_item_service", False),
    "MeterEvent": ("stripe.v2.billing._meter_event", False),
    "MeterEventAdjustment": (
        "stripe.v2.billing._meter_event_adjustment",
        False,
    ),
    "MeterEventAdjustmentService": (
        "stripe.v2.billing._meter_event_adjustment_service",
        False,
    ),
    "MeterEventService": ("stripe.v2.billing._meter_event_service", False),
    "MeterEventSession": ("stripe.v2.billing._meter_event_session", False),
    "MeterEventSessionService": (
        "stripe.v2.billing._meter_event_session_service",
        False,
    ),
    "MeterEventStreamService": (
        "stripe.v2.billing._meter_event_stream_service",
        False,
    ),
    "MeteredItem": ("stripe.v2.billing._metered_item", False),
    "MeteredItemService": ("stripe.v2.billing._metered_item_service", False),
    "OneTimeItem": ("stripe.v2.billing._one_time_item", False),
    "OneTimeItemService": ("stripe.v2.billing._one_time_item_service", False),
    "PricingPlan": ("stripe.v2.billing._pricing_plan", False),
    "PricingPlanComponent": (
        "stripe.v2.billing._pricing_plan_component",
        False,
    ),
    "PricingPlanService": ("stripe.v2.billing._pricing_plan_service", False),
    "PricingPlanSubscription": (
        "stripe.v2.billing._pricing_plan_subscription",
        False,
    ),
    "PricingPlanSubscriptionService": (
        "stripe.v2.billing._pricing_plan_subscription_service",
        False,
    ),
    "PricingPlanVersion": ("stripe.v2.billing._pricing_plan_version", False),
    "Profile": ("stripe.v2.billing._profile", False),
    "ProfileService": ("stripe.v2.billing._profile_service", False),
    "RateCard": ("stripe.v2.billing._rate_card", False),
    "RateCardCustomPricingUnitOverageRate": (
        "stripe.v2.billing._rate_card_custom_pricing_unit_overage_rate",
        False,
    ),
    "RateCardRate": ("stripe.v2.billing._rate_card_rate", False),
    "RateCardService": ("stripe.v2.billing._rate_card_service", False),
    "RateCardSubscription": (
        "stripe.v2.billing._rate_card_subscription",
        False,
    ),
    "RateCardSubscriptionService": (
        "stripe.v2.billing._rate_card_subscription_service",
        False,
    ),
    "RateCardVersion": ("stripe.v2.billing._rate_card_version", False),
    "ServiceAction": ("stripe.v2.billing._service_action", False),
    "ServiceActionService": (
        "stripe.v2.billing._service_action_service",
        False,
    ),
}
if not TYPE_CHECKING:

    def __getattr__(name):
        try:
            target, is_submodule = _import_map[name]
            module = import_module(target)
            if is_submodule:
                return module

            return getattr(
                module,
                name,
            )
        except KeyError:
            raise AttributeError()



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.params.v2.core._connection_session_create_params import (
        ConnectionSessionCreateParams,
    )
    from stripe.params.v2.core._connection_session_retrieve_params import (
        ConnectionSessionRetrieveParams,
    )
    from stripe.v2.core._connection_session import ConnectionSession


class ConnectionSessionService(StripeService):
    def create(
        self,
        params: "ConnectionSessionCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "ConnectionSession":
        """
        Create a new ConnectionSession.
        """
        return cast(
            "ConnectionSession",
            self._request(
                "post",
                "/v2/core/connection_sessions",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "ConnectionSessionCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "ConnectionSession":
        """
        Create a new ConnectionSession.
        """
        return cast(
            "ConnectionSession",
            await self._request_async(
                "post",
                "/v2/core/connection_sessions",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: Optional["ConnectionSessionRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ConnectionSession":
        """
        Retrieve a ConnectionSession.
        """
        return cast(
            "ConnectionSession",
            self._request(
                "get",
                "/v2/core/connection_sessions/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["ConnectionSessionRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ConnectionSession":
        """
        Retrieve a ConnectionSession.
        """
        return cast(
            "ConnectionSession",
            await self._request_async(
                "get",
                "/v2/core/connection_sessions/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from decimal import Decimal
from stripe._stripe_object import StripeObject, UntypedStripeObject
from stripe.v2._amount import Amount as V2AmountResource
from typing import ClassVar, List, Optional
from typing_extensions import Literal, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.v2.billing._custom_pricing_unit import (
        CustomPricingUnit as V2BillingCustomPricingUnitResource,
    )


class PricingPlanSubscription(StripeObject):
    """
    A Pricing Plan Subscription represents a customer's active subscription to a Pricing Plan. It tracks both the servicing
    status (whether the customer is receiving service) and collection status (whether payments are current). Subscriptions
    are created through Billing Intents and bill according to the associated Billing Cadence.
    """

    OBJECT_NAME: ClassVar[Literal["v2.billing.pricing_plan_subscription"]] = (
        "v2.billing.pricing_plan_subscription"
    )

    class CancellationDetails(StripeObject):
        comment: Optional[str]
        """
        Additional comments about why the user canceled the subscription, if the subscription was canceled explicitly by the user.
        """
        feedback: Optional[
            Literal[
                "customer_service",
                "low_quality",
                "missing_features",
                "other",
                "switched_service",
                "too_complex",
                "too_expensive",
                "unused",
            ]
        ]
        """
        The customer submitted reason for why they canceled, if the subscription was canceled explicitly by the user.
        """
        reason: Optional[Literal["cancellation_requested"]]
        """
        System-generated reason for cancellation.
        """

    class CollectionStatusTransitions(StripeObject):
        awaiting_customer_action_at: Optional[str]
        """
        When the collection status transitioned to awaiting customer action.
        """
        current_at: Optional[str]
        """
        When the collection status transitioned to current.
        """
        past_due_at: Optional[str]
        """
        When the collection status transitioned to past due.
        """
        paused_at: Optional[str]
        """
        When the collection status transitioned to paused.
        """
        unpaid_at: Optional[str]
        """
        When the collection status transitioned to unpaid.
        """

    class DiscountDetail(StripeObject):
        class Source(StripeObject):
            coupon: Optional[str]
            """
            The ID of the Coupon.
            """
            type: Literal["coupon"]
            """
            Type of the Discount source.
            """

        discount: str
        """
        The ID of the Discount.
        """
        end: Optional[str]
        """
        The time at which the Discount ends, if applicable.
        """
        promotion_code: Optional[str]
        """
        The ID of the PromotionCode, if applicable.
        """
        source: Source
        """
        The source of the Discount.
        """
        start: str
        """
        The time at which the Discount starts.
        """
        _inner_class_types = {"source": Source}

    class PricingPlanComponentDetail(StripeObject):
        class LicenseFeeDetails(StripeObject):
            class ServiceCycle(StripeObject):
                interval: Literal["day", "month", "week", "year"]
                """
                The interval for assessing service.
                """
                interval_count: int
                """
                The length of the interval for assessing service. For example, set this to 3 and `interval` to `"month"`
                to specify quarterly service.
                """

            class Tier(StripeObject):
                flat_amount: Optional[str]
                """
                Price for the entire tier, represented as a decimal string in minor currency units with at most 12 decimal places.
                """
                unit_amount: Optional[str]
                """
                Per-unit price for units included in this tier, represented as a decimal string in minor currency units with at
                most 12 decimal places.
                """
                up_to_decimal: Optional[Decimal]
                """
                Up to and including this quantity is contained in the tier. Only one of `up_to_decimal` and `up_to_inf` may
                be set.
                """
                up_to_inf: Optional[Literal["inf"]]
                """
                No upper bound to this tier. Only one of `up_to_decimal` and `up_to_inf` may be set.
                """
                _field_encodings = {"up_to_decimal": "decimal_string"}

            class TransformQuantity(StripeObject):
                divide_by: int
                """
                Divide usage by this number.
                """
                round: Literal["down", "up"]
                """
                After division, round the result up or down.
                """
                _field_encodings = {"divide_by": "int64_string"}

            currency: str
            """
            Three-letter ISO currency code, in lowercase.
            """
            display_name: str
            """
            A customer-facing name for the license fee.
            """
            license_fee: str
            """
            The ID of the License Fee.
            """
            license_fee_version: str
            """
            The ID of the License Fee Version.
            """
            quantity: int
            """
            Quantity of the license fee on the subscription.
            """
            service_cycle: ServiceCycle
            """
            The service cycle configuration.
            """
            tiering_mode: Optional[Literal["graduated", "volume"]]
            """
            Defines whether the tiering price is graduated or volume-based.
            """
            tiers: List[Tier]
            """
            Each element represents a pricing tier.
            """
            transform_quantity: Optional[TransformQuantity]
            """
            Apply a transformation to the reported usage or set quantity before computing the amount billed.
            """
            unit_amount: Optional[str]
            """
            The per-unit amount to be charged, represented as a decimal string in minor currency units with at most 12 decimal places.
            """
            unit_label: Optional[str]
            """
            The unit label from the licensed item, used for display purposes (for example, "seat", "environment").
            """
            _inner_class_types = {
                "service_cycle": ServiceCycle,
                "tiers": Tier,
                "transform_quantity": TransformQuantity,
            }

        class RateCardDetails(StripeObject):
            class ServiceCycle(StripeObject):
                interval: Literal["day", "month", "week", "year"]
                """
                The interval for assessing service.
                """
                interval_count: int
                """
                The length of the interval for assessing service. For example, set this to 3 and `interval` to `"month"`
                to specify quarterly service.
                """

            currency: str
            """
            Three-letter ISO currency code, in lowercase.
            """
            display_name: str
            """
            A customer-facing name for the rate card.
            """
            rate_card: str
            """
            The ID of the Rate Card.
            """
            rate_card_version: str
            """
            The ID of the Rate Card Version.
            """
            service_cycle: ServiceCycle
            """
            The service cycle configuration.
            """
            tax_behavior: Literal["exclusive", "inclusive"]
            """
            Whether the rates are inclusive or exclusive of tax.
            """
            _inner_class_types = {"service_cycle": ServiceCycle}

        class RecurringCreditGrantDetails(StripeObject):
            class CreditGrantDetails(StripeObject):
                class Amount(StripeObject):
                    class CustomPricingUnit(StripeObject):
                        custom_pricing_unit_details: Optional[
                            "V2BillingCustomPricingUnitResource"
                        ]
                        """
                        The Custom Pricing Unit object.
                        """
                        id: str
                        """
                        The id of the custom pricing unit.
                        """
                        value: Decimal
                        """
                        The value of the credit grant, decimal value represented as a string.
                        """
                        _field_encodings = {"value": "decimal_string"}

                    custom_pricing_unit: Optional[CustomPricingUnit]
                    """
                    The custom pricing unit amount of the credit grant. Required if `type` is `custom_pricing_unit`.
                    """
                    monetary: Optional[V2AmountResource]
                    """
                    The monetary amount of the credit grant. Required if `type` is `monetary`.
                    """
                    type: Literal["custom_pricing_unit", "monetary"]
                    """
                    The type of the credit grant amount. We currently support `monetary` and `custom_pricing_unit` billing credits.
                    """
                    _inner_class_types = {
                        "custom_pricing_unit": CustomPricingUnit,
                    }

                class ApplicabilityConfig(StripeObject):
                    class Scope(StripeObject):
                        billable_items: Optional[List[str]]
                        """
                        The billable items to apply the credit grant to.
                        """
                        price_type: Optional[Literal["metered"]]
                        """
                        The price type that credit grants can apply to. Stripe supports the `metered` price type, which applies to metered prices and rate cards. Cannot be used in combination with `billable_items`.
                        """

                    scope: Scope
                    """
                    The applicability scope of the credit grant.
                    """
                    _inner_class_types = {"scope": Scope}

                class ExpiryConfig(StripeObject):
                    type: Literal["end_of_service_period"]
                    """
                    The type of the expiry configuration. We currently support `end_of_service_period`.
                    """

                amount: Amount
                """
                The amount of the credit grant.
                """
                applicability_config: ApplicabilityConfig
                """
                Defines the scope where the credit grant is applicable.
                """
                expiry_config: ExpiryConfig
                """
                The expiry configuration for the credit grant.
                """
                _inner_class_types = {
                    "amount": Amount,
                    "applicability_config": ApplicabilityConfig,
                    "expiry_config": ExpiryConfig,
                }

            class CreditGrantPerTenantDetails(StripeObject):
                class Amount(StripeObject):
                    class CustomPricingUnit(StripeObject):
                        custom_pricing_unit_details: Optional[
                            "V2BillingCustomPricingUnitResource"
                        ]
                        """
                        The Custom Pricing Unit object.
                        """
                        id: str
                        """
                        The id of the custom pricing unit.
                        """
                        value: Decimal
                        """
                        The value of the credit grant, decimal value represented as a string.
                        """
                        _field_encodings = {"value": "decimal_string"}

                    custom_pricing_unit: Optional[CustomPricingUnit]
                    """
                    The custom pricing unit amount of the credit grant. Required if `type` is `custom_pricing_unit`.
                    """
                    monetary: Optional[V2AmountResource]
                    """
                    The monetary amount of the credit grant. Required if `type` is `monetary`.
                    """
                    type: Literal["custom_pricing_unit", "monetary"]
                    """
                    The type of the credit grant amount. We currently support `monetary` and `custom_pricing_unit` billing credits.
                    """
                    _inner_class_types = {
                        "custom_pricing_unit": CustomPricingUnit,
                    }

                class ApplicabilityConfig(StripeObject):
                    class Scope(StripeObject):
                        billable_items: Optional[List[str]]
                        """
                        The billable items to apply the credit grant to.
                        """
                        price_type: Optional[Literal["metered"]]
                        """
                        The price type that credit grants can apply to. Stripe supports the `metered` price type, which applies to metered prices and rate cards. Cannot be used in combination with `billable_items`.
                        """

                    scope: Scope
                    """
                    The applicability scope of the credit grant.
                    """
                    _inner_class_types = {"scope": Scope}

                class ExpiryConfig(StripeObject):
                    type: Literal["end_of_service_period"]
                    """
                    The type of the expiry configuration. We currently support `end_of_service_period`.
                    """

                amount: Amount
                """
                The amount of the credit grant.
                """
                applicability_config: ApplicabilityConfig
                """
                Defines the scope where the credit grant is applicable.
                """
                expiry_config: ExpiryConfig
                """
                The expiry configuration for the credit grant.
                """
                _inner_class_types = {
                    "amount": Amount,
                    "applicability_config": ApplicabilityConfig,
                    "expiry_config": ExpiryConfig,
                }

            class ServiceCycle(StripeObject):
                interval: Literal["day", "month", "week", "year"]
                """
                The interval for assessing service.
                """
                interval_count: int
                """
                The length of the interval for assessing service. For example, set this to 3 and `interval` to `"month"`
                to specify quarterly service.
                """

            credit_grant_details: Optional[CreditGrantDetails]
            """
            Credit grant details, present when type is CREDIT_GRANT.
            """
            credit_grant_per_tenant_details: Optional[
                CreditGrantPerTenantDetails
            ]
            """
            Credit grant per tenant details, present when type is CREDIT_GRANT_PER_TENANT.
            """
            recurring_credit_grant: str
            """
            The ID of the Recurring Credit Grant.
            """
            service_cycle: ServiceCycle
            """
            The service cycle configuration.
            """
            type: Literal["credit_grant", "credit_grant_per_tenant"]
            """
            The type of the recurring credit grant.
            """
            _inner_class_types = {
                "credit_grant_details": CreditGrantDetails,
                "credit_grant_per_tenant_details": CreditGrantPerTenantDetails,
                "service_cycle": ServiceCycle,
            }

        license_fee_details: Optional[LicenseFeeDetails]
        """
        License fee details, present when type is license_fee_details.
        """
        pricing_plan_component: str
        """
        The ID of the Pricing Plan Component.
        """
        rate_card_details: Optional[RateCardDetails]
        """
        Rate card details, present when type is rate_card_details.
        """
        recurring_credit_grant_details: Optional[RecurringCreditGrantDetails]
        """
        Recurring credit grant details, present when type is recurring_credit_grant_details.
        """
        type: Literal[
            "license_fee_details",
            "rate_card_details",
            "recurring_credit_grant_details",
        ]
        """
        The type of component details included.
        """
        _inner_class_types = {
            "license_fee_details": LicenseFeeDetails,
            "rate_card_details": RateCardDetails,
            "recurring_credit_grant_details": RecurringCreditGrantDetails,
        }

    class ServicingStatusTransitions(StripeObject):
        activated_at: Optional[str]
        """
        When the servicing status transitioned to activated.
        """
        canceled_at: Optional[str]
        """
        When the servicing status transitioned to canceled.
        """
        paused_at: Optional[str]
        """
        When the servicing status transitioned to paused.
        """
        will_activate_at: Optional[str]
        """
        When the servicing is scheduled to transition to activate.
        """
        will_cancel_at: Optional[str]
        """
        When the servicing is scheduled to cancel.
        """

    billing_cadence: str
    """
    The ID of the Billing Cadence this subscription is billed on.
    """
    cancellation_details: Optional[CancellationDetails]
    """
    Details about why the subscription was canceled, if applicable. Includes system-generated reason.
    """
    collection_status: Literal[
        "awaiting_customer_action", "current", "past_due", "paused", "unpaid"
    ]
    """
    Current collection status of this subscription.
    """
    collection_status_transitions: CollectionStatusTransitions
    """
    Timestamps for collection status transitions.
    """
    created: str
    """
    Time at which the object was created.
    """
    discount_details: Optional[List[DiscountDetail]]
    """
    Details about Discounts applied to this subscription.
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    metadata: Optional[UntypedStripeObject[str]]
    """
    Set of [key-value pairs](https://docs.stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    object: Literal["v2.billing.pricing_plan_subscription"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    pricing_plan: str
    """
    The ID of the Pricing Plan for this subscription.
    """
    pricing_plan_component_details: Optional[List[PricingPlanComponentDetail]]
    """
    Pricing plan component details for the subscription, populated when pricing_plan_component_details is included.
    """
    pricing_plan_version: str
    """
    The ID of the Pricing Plan Version for this subscription.
    """
    servicing_status: Literal["active", "canceled", "paused", "pending"]
    """
    Current servicing status of this subscription.
    """
    servicing_status_transitions: ServicingStatusTransitions
    """
    Timestamps for servicing status transitions.
    """
    test_clock: Optional[str]
    """
    The ID of the Test Clock of the associated Billing Cadence, if any.
    """
    _inner_class_types = {
        "cancellation_details": CancellationDetails,
        "collection_status_transitions": CollectionStatusTransitions,
        "discount_details": DiscountDetail,
        "pricing_plan_component_details": PricingPlanComponentDetail,
        "servicing_status_transitions": ServicingStatusTransitions,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.params.v2.billing.rate_cards._version_list_params import (
        VersionListParams,
    )
    from stripe.params.v2.billing.rate_cards._version_retrieve_params import (
        VersionRetrieveParams,
    )
    from stripe.v2._list_object import ListObject
    from stripe.v2.billing._rate_card_version import RateCardVersion


class VersionService(StripeService):
    def list(
        self,
        rate_card_id: str,
        params: Optional["VersionListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[RateCardVersion]":
        """
        List the versions of a Rate Card object. Results are sorted in reverse chronological order (most recent first).
        """
        return cast(
            "ListObject[RateCardVersion]",
            self._request(
                "get",
                "/v2/billing/rate_cards/{rate_card_id}/versions".format(
                    rate_card_id=sanitize_id(rate_card_id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        rate_card_id: str,
        params: Optional["VersionListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[RateCardVersion]":
        """
        List the versions of a Rate Card object. Results are sorted in reverse chronological order (most recent first).
        """
        return cast(
            "ListObject[RateCardVersion]",
            await self._request_async(
                "get",
                "/v2/billing/rate_cards/{rate_card_id}/versions".format(
                    rate_card_id=sanitize_id(rate_card_id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        rate_card_id: str,
        id: str,
        params: Optional["VersionRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "RateCardVersion":
        """
        Retrieve a specific version of a Rate Card object.
        """
        return cast(
            "RateCardVersion",
            self._request(
                "get",
                "/v2/billing/rate_cards/{rate_card_id}/versions/{id}".format(
                    rate_card_id=sanitize_id(rate_card_id),
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        rate_card_id: str,
        id: str,
        params: Optional["VersionRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "RateCardVersion":
        """
        Retrieve a specific version of a Rate Card object.
        """
        return cast(
            "RateCardVersion",
            await self._request_async(
                "get",
                "/v2/billing/rate_cards/{rate_card_id}/versions/{id}".format(
                    rate_card_id=sanitize_id(rate_card_id),
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing_extensions import NotRequired, TypedDict


class QueryRunCreateParams(TypedDict):
    result_options: NotRequired["QueryRunCreateParamsResultOptions"]
    """
    Optional settings to customize the results of the `QueryRun`.
    """
    sql: str
    """
    The SQL to execute.
    """


class QueryRunCreateParamsResultOptions(TypedDict):
    compress_file: NotRequired[bool]
    """
    If set, the generated results file will be compressed into a ZIP folder.
    This is useful for reducing file size and download time for large results.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe.params.treasury._transaction_entry_list_params import (
        TransactionEntryListParams,
    )
    from stripe.params.treasury._transaction_entry_retrieve_params import (
        TransactionEntryRetrieveParams,
    )
    from stripe.treasury._transaction_entry import TransactionEntry


class TransactionEntryService(StripeService):
    def list(
        self,
        params: "TransactionEntryListParams",
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[TransactionEntry]":
        """
        Retrieves a list of TransactionEntry objects.
        """
        return cast(
            "ListObject[TransactionEntry]",
            self._request(
                "get",
                "/v1/treasury/transaction_entries",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: "TransactionEntryListParams",
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[TransactionEntry]":
        """
        Retrieves a list of TransactionEntry objects.
        """
        return cast(
            "ListObject[TransactionEntry]",
            await self._request_async(
                "get",
                "/v1/treasury/transaction_entries",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: Optional["TransactionEntryRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "TransactionEntry":
        """
        Retrieves a TransactionEntry object.
        """
        return cast(
            "TransactionEntry",
            self._request(
                "get",
                "/v1/treasury/transaction_entries/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["TransactionEntryRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "TransactionEntry":
        """
        Retrieves a TransactionEntry object.
        """
        return cast(
            "TransactionEntry",
            await self._request_async(
                "get",
                "/v1/treasury/transaction_entries/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from decimal import Decimal
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, List
from typing_extensions import Literal, NotRequired, TypedDict


class RateCardModifyRatesParams(TypedDict):
    rates_to_create: List["RateCardModifyRatesParamsRatesToCreate"]
    """
    The list of RateCard rates to create or update. Maximum of 100 rates.
    """
    rates_to_delete: List["RateCardModifyRatesParamsRatesToDelete"]
    """
    The list of RateCard rates to delete. Maximum of 100 rates.
    """


class RateCardModifyRatesParamsRatesToCreate(TypedDict):
    metadata: NotRequired["Dict[str, str]|UntypedStripeObject[str]"]
    """
    Set of [key-value pairs](https://docs.stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    metered_item: NotRequired[str]
    """
    The Metered Item that this rate binds to. Cannot be set if `metered_item_data` is provided, and must be set if it isn't.
    """
    metered_item_data: NotRequired[
        "RateCardModifyRatesParamsRatesToCreateMeteredItemData"
    ]
    """
    The data to create a metered item that binds to this rate. Cannot be set if `metered_item` is provided, and must be set if it isn't.
    """
    tiering_mode: NotRequired[Literal["graduated", "volume"]]
    """
    Defines whether the tiered price is graduated or volume-based. In volume-based tiering, the maximum
    quantity within a period determines the per-unit price. In graduated tiering, the pricing changes as the quantity
    grows into new tiers. Can only be set if `tiers` is set.
    """
    tiers: NotRequired[List["RateCardModifyRatesParamsRatesToCreateTier"]]
    """
    Each element represents a pricing tier. Cannot be set if `unit_amount` is provided.
    """
    transform_quantity: NotRequired[
        "RateCardModifyRatesParamsRatesToCreateTransformQuantity"
    ]
    """
    Apply a transformation to the reported usage or set quantity before computing the amount billed.
    """
    unit_amount: NotRequired[str]
    """
    The per-unit amount to be charged, represented as a decimal string in minor currency units with at most 12 decimal
    places. Cannot be set if `tiers` is provided.
    """


class RateCardModifyRatesParamsRatesToCreateMeteredItemData(TypedDict):
    display_name: str
    """
    Description that customers see in the invoice line item.
    Maximum length of 250 characters.
    """
    lookup_key: NotRequired[str]
    """
    An internal key you can use to search for a particular metered item.
    Must be unique among metered items.
    Maximum length of 200 characters.
    """
    meter: str
    """
    ID of the Meter that measures usage for this Metered Item.
    """
    meter_segment_conditions: List[
        "RateCardModifyRatesParamsRatesToCreateMeteredItemDataMeterSegmentCondition"
    ]
    """
    Optional array of Meter segments to filter event dimension keys for billing.
    """
    unit_label: NotRequired[str]
    """
    The unit to use when displaying prices for this billable item in places like Checkout. For example, set this field
    to "CPU-hour" for Checkout to display "(price) per CPU-hour", or "1 million events" to display "(price) per 1
    million events".
    Maximum length of 100 characters.
    """


class RateCardModifyRatesParamsRatesToCreateMeteredItemDataMeterSegmentCondition(
    TypedDict,
):
    dimension: str
    """
    A Meter dimension.
    """
    value: str
    """
    To count usage towards this metered item, the dimension must have this value.
    """


class RateCardModifyRatesParamsRatesToCreateTier(TypedDict):
    flat_amount: NotRequired[str]
    """
    Price for the entire tier, represented as a decimal string in minor currency units with at most 12 decimal places.
    """
    unit_amount: NotRequired[str]
    """
    Per-unit price for units included in this tier, represented as a decimal string in minor currency units with at
    most 12 decimal places.
    """
    up_to_decimal: NotRequired[Decimal]
    """
    Up to and including this quantity is contained in the tier. Only one of `up_to_decimal` and `up_to_inf` may
    be set.
    """
    up_to_inf: NotRequired[Literal["inf"]]
    """
    No upper bound to this tier. Only one of `up_to_decimal` and `up_to_inf` may be set.
    """


class RateCardModifyRatesParamsRatesToCreateTransformQuantity(TypedDict):
    divide_by: int
    """
    Divide usage by this number.
    """
    round: Literal["down", "up"]
    """
    After division, round the result up or down.
    """


class RateCardModifyRatesParamsRatesToDelete(TypedDict):
    id: str
    """
    The ID of the RateCard rate to delete.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.params.v2.test_helpers._financial_address_credit_params import (
        FinancialAddressCreditParams,
    )
    from stripe.params.v2.test_helpers._financial_address_generate_microdeposits_params import (
        FinancialAddressGenerateMicrodepositsParams,
    )
    from stripe.v2._financial_address_credit_simulation import (
        FinancialAddressCreditSimulation,
    )
    from stripe.v2._financial_address_generated_microdeposits import (
        FinancialAddressGeneratedMicrodeposits,
    )


class FinancialAddressService(StripeService):
    def credit(
        self,
        id: str,
        params: "FinancialAddressCreditParams",
        options: Optional["RequestOptions"] = None,
    ) -> "FinancialAddressCreditSimulation":
        """
        Simulate crediting a FinancialAddress in a Sandbox environment. This can be used to add virtual funds and increase your balance for testing.
        """
        return cast(
            "FinancialAddressCreditSimulation",
            self._request(
                "post",
                "/v2/test_helpers/financial_addresses/{id}/credit".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def credit_async(
        self,
        id: str,
        params: "FinancialAddressCreditParams",
        options: Optional["RequestOptions"] = None,
    ) -> "FinancialAddressCreditSimulation":
        """
        Simulate crediting a FinancialAddress in a Sandbox environment. This can be used to add virtual funds and increase your balance for testing.
        """
        return cast(
            "FinancialAddressCreditSimulation",
            await self._request_async(
                "post",
                "/v2/test_helpers/financial_addresses/{id}/credit".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def generate_microdeposits(
        self,
        id: str,
        params: Optional["FinancialAddressGenerateMicrodepositsParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "FinancialAddressGeneratedMicrodeposits":
        """
        Generates microdeposits for a FinancialAddress in a Sandbox environment.
        """
        return cast(
            "FinancialAddressGeneratedMicrodeposits",
            self._request(
                "post",
                "/v2/test_helpers/financial_addresses/{id}/generate_microdeposits".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def generate_microdeposits_async(
        self,
        id: str,
        params: Optional["FinancialAddressGenerateMicrodepositsParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "FinancialAddressGeneratedMicrodeposits":
        """
        Generates microdeposits for a FinancialAddress in a Sandbox environment.
        """
        return cast(
            "FinancialAddressGeneratedMicrodeposits",
            await self._request_async(
                "post",
                "/v2/test_helpers/financial_addresses/{id}/generate_microdeposits".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject
from stripe.v2._amount import Amount
from typing import ClassVar, Optional
from typing_extensions import Literal


class ReceivedCredit(StripeObject):
    """
    Use ReceivedCredits API to retrieve information on when, where, and how funds are sent into your FinancialAccount.
    """

    OBJECT_NAME: ClassVar[Literal["v2.money_management.received_credit"]] = (
        "v2.money_management.received_credit"
    )

    class BalanceTransfer(StripeObject):
        from_account: Optional[str]
        """
        The ID of the account that owns the source object originated the ReceivedCredit.
        """
        outbound_payment: Optional[str]
        """
        The ID of the outbound payment object that originated the ReceivedCredit.
        """
        outbound_transfer: Optional[str]
        """
        The ID of the outbound transfer object that originated the ReceivedCredit.
        """
        payout_v1: Optional[str]
        """
        The ID of the payout object that originated the ReceivedCredit.
        """
        transfer: Optional[str]
        """
        The ID of the v1 transfer object that originated the ReceivedCredit.
        """
        type: Literal[
            "outbound_payment", "outbound_transfer", "transfer", "payout_v1"
        ]
        """
        Open Enum. The type of Stripe Money Movement that originated the ReceivedCredit.
        """

    class BankTransfer(StripeObject):
        class CaBankAccount(StripeObject):
            account_holder_name: Optional[str]
            """
            The account holder name of the bank account the transfer was received from.
            """
            bank_name: Optional[str]
            """
            The bank name the transfer was received from.
            """
            last4: Optional[str]
            """
            The last 4 digits of the account number that originated the transfer.
            Depending on the bank, this may instead be the last 4 digits of the return account number.
            """
            network: Literal["acss"]
            """
            Open Enum. The money transmission network used to send funds for this ReceivedCredit.
            """

        class EuBankAccount(StripeObject):
            account_holder_name: Optional[str]
            """
            The account holder name of the bank account the transfer was received from.
            """
            bank_name: Optional[str]
            """
            The bank name the transfer was received from.
            """
            bic: Optional[str]
            """
            The bic of the account that originated the transfer.
            """
            last4: Optional[str]
            """
            The last 4 digits of the account number that originated the transfer.
            """
            network: Literal["sepa"]
            """
            Open Enum. The money transmission network used to send funds for this ReceivedCredit.
            """

        class GbBankAccount(StripeObject):
            account_holder_name: Optional[str]
            """
            The bank name the transfer was received from.
            """
            bank_name: Optional[str]
            """
            The bank name the transfer was received from.
            """
            last4: Optional[str]
            """
            The last 4 digits of the account number that originated the transfer.
            """
            network: Literal["chaps", "fps"]
            """
            Open Enum. The money transmission network used to send funds for this ReceivedCredit.
            """
            sort_code: Optional[str]
            """
            The sort code of the account that originated the transfer.
            """

        class MxBankAccount(StripeObject):
            account_holder_name: Optional[str]
            """
            The account holder name of the bank account the transfer was received from.
            """
            bank_name: Optional[str]
            """
            The bank name the transfer was received from.
            """
            last4: Optional[str]
            """
            The last 4 digits of the account number that originated the transfer.
            """
            network: Literal["spei"]
            """
            Open Enum. The money transmission network used to send funds for this ReceivedCredit.
            """

        class SepaBankAccount(StripeObject):
            account_holder_name: Optional[str]
            """
            The account holder name of the bank account the transfer was received from.
            """
            bank_name: Optional[str]
            """
            The bank name the transfer was received from.
            """
            bic: Optional[str]
            """
            The BIC of the SEPA account.
            """
            country: Optional[str]
            """
            The origination country of the bank transfer.
            """
            iban: Optional[str]
            """
            The IBAN that originated the transfer.
            """
            network: Literal["sepa_credit_transfer"]
            """
            The money transmission network used to send funds for this ReceivedCredit.
            """

        class UsBankAccount(StripeObject):
            account_holder_name: Optional[str]
            """
            The name of the account holder that sent the payment.
            """
            bank_name: Optional[str]
            """
            The bank name the transfer was received from.
            """
            last4: Optional[str]
            """
            The last 4 digits of the account number that originated the transfer.
            """
            network: Literal["ach", "rtp", "us_domestic_wire"]
            """
            Open Enum. The money transmission network used to send funds for this ReceivedCredit.
            """
            routing_number: Optional[str]
            """
            The routing number of the account that originated the transfer.
            """

        ca_bank_account: Optional[CaBankAccount]
        """
        Hash containing the transaction bank details. Present if `origin_type` field value is `ca_bank_account`.
        """
        eu_bank_account: Optional[EuBankAccount]
        """
        Hash containing the transaction bank details. Present if `origin_type` field value is `eu_bank_account`.
        """
        financial_address: str
        """
        Financial Address on which funds for ReceivedCredit were received.
        """
        gb_bank_account: Optional[GbBankAccount]
        """
        Hash containing the transaction bank details. Present if `origin_type` field value is `gb_bank_account`.
        """
        mx_bank_account: Optional[MxBankAccount]
        """
        Hash containing the transaction bank details. Present if  `origin_type` field value is `mx_bank_account`.
        """
        origin_type: Literal[
            "ca_bank_account",
            "crypto_wallet",
            "eu_bank_account",
            "gb_bank_account",
            "mx_bank_account",
            "sepa_bank_account",
            "us_bank_account",
        ]
        """
        Open Enum. Indicates the origin of source from which external funds originated from.
        """
        sepa_bank_account: Optional[SepaBankAccount]
        """
        Hash containing the transaction bank details. Present if `origin_type` field value is `sepa_bank_account`.
        """
        statement_descriptor: Optional[str]
        """
        Freeform string set by originator of the external ReceivedCredit.
        """
        us_bank_account: Optional[UsBankAccount]
        """
        Hash containing the transaction bank details. Present if `origin_type` field value is `us_bank_account`.
        """
        _inner_class_types = {
            "ca_bank_account": CaBankAccount,
            "eu_bank_account": EuBankAccount,
            "gb_bank_account": GbBankAccount,
            "mx_bank_account": MxBankAccount,
            "sepa_bank_account": SepaBankAccount,
            "us_bank_account": UsBankAccount,
        }

    class CardSpend(StripeObject):
        class Dispute(StripeObject):
            issuing_dispute_v1: str
            """
            The reference to the v1 issuing dispute ID.
            """

        class Refund(StripeObject):
            issuing_transaction_v1: str
            """
            The reference to the v1 issuing transaction ID.
            """

        card_v1_id: str
        """
        The reference to the issuing card object.
        """
        dispute: Optional[Dispute]
        """
        Hash containing information about the Dispute that triggered this credit.
        """
        refund: Optional[Refund]
        """
        Hash containing information about the Refund that triggered this credit.
        """
        _inner_class_types = {"dispute": Dispute, "refund": Refund}

    class CryptoWalletTransfer(StripeObject):
        class CryptoWallet(StripeObject):
            address: str
            """
            The address of the wallet the crypto was received from.
            """
            memo: str
            """
            A memo also for identifying the recipient for memo-based blockchains (e.g., Stellar),.
            """
            network: Literal[
                "arbitrum",
                "avalanche_c_chain",
                "base",
                "ethereum",
                "optimism",
                "polygon",
                "solana",
                "stellar",
                "tempo",
            ]
            """
            The network the crypto was received from.
            """

        crypto_wallet: CryptoWallet
        """
        Hash containing the transaction crypto wallet details.
        """
        financial_address: str
        """
        Financial Address on which funds for ReceivedCredit were received.
        """
        origin_type: Literal[
            "ca_bank_account",
            "crypto_wallet",
            "eu_bank_account",
            "gb_bank_account",
            "mx_bank_account",
            "sepa_bank_account",
            "us_bank_account",
        ]
        """
        Open Enum. Indicates the origin of source from which external funds originated from.
        """
        statement_descriptor: Optional[str]
        """
        Freeform string set by originator of the external ReceivedCredit.
        """
        _inner_class_types = {"crypto_wallet": CryptoWallet}

    class StatusDetails(StripeObject):
        class Failed(StripeObject):
            reason: Literal[
                "capability_inactive",
                "currency_unsupported_on_financial_address",
                "financial_address_inactive",
                "stripe_rejected",
            ]
            """
            Open Enum. The `failed` status reason.
            """

        class Returned(StripeObject):
            reason: Literal["originator_initiated_reversal"]
            """
            Open Enum. The `returned` status reason.
            """

        failed: Optional[Failed]
        """
        Hash that provides additional information regarding the reason behind a `failed` ReceivedCredit status. It is only present when the ReceivedCredit status is `failed`.
        """
        returned: Optional[Returned]
        """
        Hash that provides additional information regarding the reason behind a `returned` ReceivedCredit status. It is only present when the ReceivedCredit status is `returned`.
        """
        _inner_class_types = {"failed": Failed, "returned": Returned}

    class StatusTransitions(StripeObject):
        failed_at: Optional[str]
        """
        Timestamp describing when the ReceivedCredit was marked as `failed`.
        Represented as a RFC 3339 date & time UTC value in millisecond precision, for example: 2022-09-18T13:22:18.123Z.
        """
        returned_at: Optional[str]
        """
        Timestamp describing when the ReceivedCredit changed status to `returned`.
        Represented as a RFC 3339 date & time UTC value in millisecond precision, for example: 2022-09-18T13:22:18.123Z.
        """
        succeeded_at: Optional[str]
        """
        Timestamp describing when the ReceivedCredit was marked as `succeeded`.
        Represented as a RFC 3339 date & time UTC value in millisecond precision, for example: 2022-09-18T13:22:18.123Z.
        """

    class StripeBalancePayment(StripeObject):
        debit_agreement: Optional[str]
        """
        ID of the debit agreement associated with this payment.
        """
        statement_descriptor: Optional[str]
        """
        Statement descriptor for the Stripe Balance Payment.
        """

    amount: Amount
    """
    The amount and currency of the ReceivedCredit.
    """
    balance_transfer: Optional[BalanceTransfer]
    """
    This object stores details about the originating Stripe transaction that resulted in the ReceivedCredit. Present if `type` field value is `balance_transfer`.
    """
    bank_transfer: Optional[BankTransfer]
    """
    This object stores details about the originating banking transaction that resulted in the ReceivedCredit. Present if `type` field value is `bank_transfer`.
    """
    card_spend: Optional[CardSpend]
    """
    This object stores details about the originating issuing card spend that resulted in the ReceivedCredit. Present if `type` field value is `card_spend`.
    """
    created: str
    """
    Time at which the ReceivedCredit was created.
    Represented as a RFC 3339 date & time UTC value in millisecond precision, for example: 2022-09-18T13:22:18.123Z.
    """
    crypto_wallet_transfer: Optional[CryptoWalletTransfer]
    """
    This object stores details about the originating crypto transaction that resulted in the ReceivedCredit. Present if `type` field value is `crypto_wallet_transfer`.
    """
    description: Optional[str]
    """
    Freeform string set by originator of the ReceivedCredit.
    """
    external_amount: Optional[Amount]
    """
    The amount and currency of the original/external credit request.
    """
    financial_account: str
    """
    Financial Account ID on which funds for ReceivedCredit were received.
    """
    id: str
    """
    Unique identifier for the ReceivedCredit.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    object: Literal["v2.money_management.received_credit"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    receipt_url: Optional[str]
    """
    A hosted transaction receipt URL that is provided when money movement is considered regulated under Stripe's money transmission licenses.
    """
    status: Literal["failed", "pending", "returned", "succeeded"]
    """
    Open Enum. The status of the ReceivedCredit.
    """
    status_details: Optional[StatusDetails]
    """
    This hash contains detailed information that elaborates on the specific status of the ReceivedCredit. e.g the reason behind a failure if the status is marked as `failed`.
    """
    status_transitions: Optional[StatusTransitions]
    """
    Hash containing timestamps of when the object transitioned to a particular status.
    """
    stripe_balance_payment: Optional[StripeBalancePayment]
    """
    This object stores details about the stripe balance pay refund that resulted in the ReceivedCredit. Present if `type` field value is `stripe_balance_payment`.
    """
    type: Literal[
        "balance_transfer",
        "bank_transfer",
        "card_spend",
        "crypto_wallet_transfer",
        "external_credit",
        "stripe_balance_payment",
    ]
    """
    Open Enum. The type of flow that caused the ReceivedCredit.
    """
    _inner_class_types = {
        "balance_transfer": BalanceTransfer,
        "bank_transfer": BankTransfer,
        "card_spend": CardSpend,
        "crypto_wallet_transfer": CryptoWalletTransfer,
        "status_details": StatusDetails,
        "status_transitions": StatusTransitions,
        "stripe_balance_payment": StripeBalancePayment,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject
from typing import ClassVar, Optional
from typing_extensions import Literal


class Intent(StripeObject):
    """
    A Billing Intent represents a proposed change to a customer's billing configuration, such as subscribing to a new service,
    modifying an existing subscription, or canceling service. Intents follow a draft-reserve-commit workflow where they can be
    previewed before committing, allowing you to see the billing impact before changes take effect.
    """

    OBJECT_NAME: ClassVar[Literal["v2.billing.intent"]] = "v2.billing.intent"

    class AmountDetails(StripeObject):
        currency: str
        """
        Three-letter ISO currency code, in lowercase. Must be a supported currency.
        """
        discount: str
        """
        Amount of discount applied.
        """
        shipping: str
        """
        Amount of shipping charges.
        """
        subtotal: str
        """
        Subtotal amount before tax and discounts.
        """
        tax: str
        """
        Amount of tax.
        """
        total: str
        """
        Total amount for the Billing Intent.
        """

    class CadenceData(StripeObject):
        class BillingCycle(StripeObject):
            class Day(StripeObject):
                class Time(StripeObject):
                    hour: int
                    """
                    The hour at which the billing cycle ends.
                    This must be an integer between 0 and 23, inclusive.
                    0 represents midnight, and 23 represents 11 PM.
                    """
                    minute: int
                    """
                    The minute at which the billing cycle ends.
                    Must be an integer between 0 and 59, inclusive.
                    """
                    second: Optional[int]
                    """
                    The second at which the billing cycle ends.
                    Must be an integer between 0 and 59, inclusive.
                    """

                time: Time
                """
                The time at which the billing cycle ends.
                """
                _inner_class_types = {"time": Time}

            class Month(StripeObject):
                class Time(StripeObject):
                    hour: int
                    """
                    The hour at which the billing cycle ends.
                    This must be an integer between 0 and 23, inclusive.
                    0 represents midnight, and 23 represents 11 PM.
                    """
                    minute: int
                    """
                    The minute at which the billing cycle ends.
                    Must be an integer between 0 and 59, inclusive.
                    """
                    second: Optional[int]
                    """
                    The second at which the billing cycle ends.
                    Must be an integer between 0 and 59, inclusive.
                    """

                day_of_month: int
                """
                The day to anchor the billing on for a type="month" billing cycle from 1-31.
                If this number is greater than the number of days in the month being billed,
                this anchors to the last day of the month.
                """
                month_of_year: Optional[int]
                """
                The month to anchor the billing on for a type="month" billing cycle from
                1-12. Occurrences are calculated from the month anchor.
                """
                time: Time
                """
                The time at which the billing cycle ends.
                """
                _inner_class_types = {"time": Time}

            class Week(StripeObject):
                class Time(StripeObject):
                    hour: int
                    """
                    The hour at which the billing cycle ends.
                    This must be an integer between 0 and 23, inclusive.
                    0 represents midnight, and 23 represents 11 PM.
                    """
                    minute: int
                    """
                    The minute at which the billing cycle ends.
                    Must be an integer between 0 and 59, inclusive.
                    """
                    second: Optional[int]
                    """
                    The second at which the billing cycle ends.
                    Must be an integer between 0 and 59, inclusive.
                    """

                day_of_week: int
                """
                The day of the week to bill the type=week billing cycle on.
                Numbered from 1-7 for Monday to Sunday respectively, based on the ISO-8601 week day numbering.
                """
                time: Time
                """
                The time at which the billing cycle ends.
                """
                _inner_class_types = {"time": Time}

            class Year(StripeObject):
                class Time(StripeObject):
                    hour: int
                    """
                    The hour at which the billing cycle ends.
                    This must be an integer between 0 and 23, inclusive.
                    0 represents midnight, and 23 represents 11 PM.
                    """
                    minute: int
                    """
                    The minute at which the billing cycle ends.
                    Must be an integer between 0 and 59, inclusive.
                    """
                    second: Optional[int]
                    """
                    The second at which the billing cycle ends.
                    Must be an integer between 0 and 59, inclusive.
                    """

                day_of_month: int
                """
                The day to anchor the billing on for a type="month" billing cycle from 1-31.
                If this number is greater than the number of days in the month being billed,
                this anchors to the last day of the month.
                """
                month_of_year: int
                """
                The month to bill on from 1-12. If not provided, this defaults to the month the cadence was created.
                """
                time: Time
                """
                The time at which the billing cycle ends.
                """
                _inner_class_types = {"time": Time}

            day: Optional[Day]
            """
            Specific configuration for determining billing dates when type=day.
            """
            interval_count: int
            """
            The number of intervals (specified in the interval attribute) between cadence billings. For example, type=month and interval_count=3 bills every 3 months.
            """
            month: Optional[Month]
            """
            Specific configuration for determining billing dates when type=month.
            """
            type: Literal["day", "month", "week", "year"]
            """
            The frequency at which a cadence bills.
            """
            week: Optional[Week]
            """
            Specific configuration for determining billing dates when type=week.
            """
            year: Optional[Year]
            """
            Specific configuration for determining billing dates when type=year.
            """
            _inner_class_types = {
                "day": Day,
                "month": Month,
                "week": Week,
                "year": Year,
            }

        class Payer(StripeObject):
            class BillingProfileData(StripeObject):
                customer: str
                """
                The customer to associate with the profile.
                """
                default_payment_method: Optional[str]
                """
                The default payment method to use when billing this profile.
                If none is provided, the customer `default_payment_method` is used.
                """

            billing_profile: Optional[str]
            """
            The ID of the Billing Profile object which determines how a bill is paid.
            """
            billing_profile_data: Optional[BillingProfileData]
            """
            Data for creating a new profile.
            """
            _inner_class_types = {"billing_profile_data": BillingProfileData}

        class Settings(StripeObject):
            class Bill(StripeObject):
                id: str
                """
                The ID of the referenced settings object.
                """
                version: Optional[str]
                """
                Returns the Settings Version when the cadence is pinned to a specific version.
                """

            class Collection(StripeObject):
                id: str
                """
                The ID of the referenced settings object.
                """
                version: Optional[str]
                """
                Returns the Settings Version when the cadence is pinned to a specific version.
                """

            bill: Optional[Bill]
            """
            Settings that configure bills generation, which includes calculating totals, tax, and presenting invoices.
            """
            collection: Optional[Collection]
            """
            Settings that configure and manage the behavior of collecting payments.
            """
            _inner_class_types = {"bill": Bill, "collection": Collection}

        billing_cycle: BillingCycle
        """
        The billing cycle configuration for this Cadence.
        """
        payer: Payer
        """
        Information about the payer for this Cadence.
        """
        settings: Optional[Settings]
        """
        Settings for creating the Cadence.
        """
        _inner_class_types = {
            "billing_cycle": BillingCycle,
            "payer": Payer,
            "settings": Settings,
        }

    class StatusTransitions(StripeObject):
        canceled_at: Optional[str]
        """
        Time at which the Billing Intent was canceled.
        """
        committed_at: Optional[str]
        """
        Time at which the Billing Intent was committed.
        """
        drafted_at: Optional[str]
        """
        Time at which the Billing Intent was drafted.
        """
        expires_at: str
        """
        Time at which the Billing Intent expires.
        """
        reserved_at: Optional[str]
        """
        Time at which the Billing Intent was reserved.
        """

    amount_details: AmountDetails
    """
    Breakdown of the amount for this Billing Intent.
    """
    cadence: Optional[str]
    """
    ID of an existing Cadence to use.
    """
    cadence_data: Optional[CadenceData]
    """
    Data for creating a new Cadence.
    """
    created: str
    """
    Time at which the object was created.
    """
    currency: str
    """
    Three-letter ISO currency code, in lowercase. Must be a supported currency.
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    object: Literal["v2.billing.intent"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    status: Literal["canceled", "committed", "draft", "reserved"]
    """
    Current status of the Billing Intent.
    """
    status_transitions: StatusTransitions
    """
    Timestamps for status transitions of the Billing Intent.
    """
    _inner_class_types = {
        "amount_details": AmountDetails,
        "cadence_data": CadenceData,
        "status_transitions": StatusTransitions,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject
from typing import ClassVar, Optional
from typing_extensions import Literal


class Profile(StripeObject):
    """
    A Stripe profile
    """

    OBJECT_NAME: ClassVar[Literal["profile"]] = "profile"

    class Branding(StripeObject):
        class Icon(StripeObject):
            original: str

        class Logo(StripeObject):
            original: str

        icon: Optional[Icon]
        logo: Optional[Logo]
        primary_color: Optional[str]
        secondary_color: Optional[str]
        _inner_class_types = {"icon": Icon, "logo": Logo}

    branding: Optional[Branding]
    description: Optional[str]
    display_name: str
    id: str
    livemode: bool
    object: Literal["profile"]
    url: Optional[str]
    username: str
    _inner_class_types = {"branding": Branding}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject
from typing import ClassVar
from typing_extensions import Literal


class RiskSignals(StripeObject):
    OBJECT_NAME: ClassVar[Literal["risk_signals"]] = "risk_signals"
    object: Literal["risk_signals"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    session_metadata: bool
    """
    Represents the status of risk signal session metadata collection. When false, the account has payouts and payments disabled.
    """



# -*- coding: utf-8 -*-
# File copied from our code generator; changes here will be overwritten.
import json
import pytest
from typing import Optional
from unittest.mock import Mock

from stripe import StripeClient
from stripe._event_notification_handler import (
    StripeEventNotificationHandler,
    UnhandledNotificationDetails,
)
from stripe._stripe_context import StripeContext
from stripe.events._v1_billing_meter_error_report_triggered_event import (
    V1BillingMeterErrorReportTriggeredEventNotification,
)
from stripe.events._v2_core_account_created_event import (
    V2CoreAccountCreatedEventNotification,
)
from stripe.v2.core._event import EventNotification, UnknownEventNotification
from tests.http_client_mock import HTTPClientMock
from tests.test_webhook import DUMMY_WEBHOOK_SECRET, generate_header


class TestEventNotificationHandler:
    @pytest.fixture(scope="function")
    def stripe_client(self, http_client_mock: HTTPClientMock) -> StripeClient:
        return StripeClient(
            api_key="sk_test_1234",
            stripe_context=StripeContext.parse("original_context_123"),
            http_client=http_client_mock.get_mock_http_client(),
        )

    @pytest.fixture(scope="function")
    def fallback_callback(self) -> Mock:
        """Mock handler for unhandled events"""
        return Mock()

    @pytest.fixture(scope="function")
    def event_handler(
        self, stripe_client: StripeClient, fallback_callback: Mock
    ) -> StripeEventNotificationHandler:
        return StripeEventNotificationHandler(
            client=stripe_client,
            webhook_secret=DUMMY_WEBHOOK_SECRET,
            fallback_callback=fallback_callback,
        )

    @pytest.fixture(scope="function")
    def v1_billing_meter_payload(self) -> str:
        """A payload for v1.billing.meter.error_report_triggered event"""
        return json.dumps(
            {
                "id": "evt_123",
                "object": "v2.core.event",
                "type": "v1.billing.meter.error_report_triggered",
                "livemode": False,
                "created": "2022-02-15T00:27:45.330Z",
                "context": "event_context_456",
                "related_object": {
                    "id": "mtr_123",
                    "type": "billing.meter",
                    "url": "/v1/billing/meters/mtr_123",
                },
            }
        )

    @pytest.fixture(scope="function")
    def v2_account_created_payload(self) -> str:
        """A payload for v2.core.account.created event with None context"""
        return json.dumps(
            {
                "id": "evt_789",
                "object": "v2.core.event",
                "type": "v2.core.account.created",
                "livemode": False,
                "created": "2022-02-15T00:27:45.330Z",
                "context": None,
                "related_object": {
                    "id": "acct_abc",
                    "type": "account",
                    "url": "/v2/core/accounts/acct_abc",
                },
            }
        )

    @pytest.fixture(scope="function")
    def unknown_event_payload(self) -> str:
        """A payload for an unknown event type (llama.created)"""
        return json.dumps(
            {
                "id": "evt_unknown",
                "object": "v2.core.event",
                "type": "llama.created",
                "livemode": False,
                "created": "2022-02-15T00:27:45.330Z",
                "context": "event_context_unknown",
                "related_object": {
                    "id": "llama_123",
                    "type": "llama",
                    "url": "/v1/llamas/llama_123",
                },
            }
        )

    def test_routes_event_to_registered_handler(
        self,
        event_handler: StripeEventNotificationHandler,
        v1_billing_meter_payload: str,
        fallback_callback: Mock,
    ) -> None:
        """Test that a registered event type is routed to the correct handler"""
        handler = Mock()
        event_handler.on_v1_billing_meter_error_report_triggered(handler)

        sig_header = generate_header(payload=v1_billing_meter_payload)
        event_handler.handle(v1_billing_meter_payload, sig_header)

        handler.assert_called_once()

        call_args = handler.call_args[0]
        assert isinstance(
            call_args[0], V1BillingMeterErrorReportTriggeredEventNotification
        )

        fallback_callback.assert_not_called()

    def test_routes_different_events_to_correct_handlers(
        self,
        event_handler: StripeEventNotificationHandler,
        v1_billing_meter_payload: str,
        v2_account_created_payload: str,
        fallback_callback: Mock,
    ) -> None:
        """Test that different event types route to their respective handlers"""
        billing_handler = Mock()
        account_handler = Mock()

        event_handler.on_v1_billing_meter_error_report_triggered(
            billing_handler
        )
        event_handler.on_v2_core_account_created(account_handler)

        sig_header1 = generate_header(payload=v1_billing_meter_payload)
        event_handler.handle(v1_billing_meter_payload, sig_header1)

        sig_header2 = generate_header(payload=v2_account_created_payload)
        event_handler.handle(v2_account_created_payload, sig_header2)

        billing_handler.assert_called_once()
        account_handler.assert_called_once()

        assert isinstance(
            billing_handler.call_args[0][0],
            V1BillingMeterErrorReportTriggeredEventNotification,
        )
        assert isinstance(
            account_handler.call_args[0][0],
            V2CoreAccountCreatedEventNotification,
        )

        fallback_callback.assert_not_called()

    def test_handler_receives_correct_runtime_type(
        self,
        event_handler: StripeEventNotificationHandler,
        v1_billing_meter_payload: str,
    ) -> None:
        """Test that handlers receive the correctly typed event notification"""
        received_event: Optional[EventNotification] = None
        received_client: Optional[StripeClient] = None

        def handler(
            event: V1BillingMeterErrorReportTriggeredEventNotification,
            client: StripeClient,
        ) -> None:
            nonlocal received_event, received_client
            received_event = event
            received_client = client

        event_handler.on_v1_billing_meter_error_report_triggered(handler)

        sig_header = generate_header(payload=v1_billing_meter_payload)
        event_handler.handle(v1_billing_meter_payload, sig_header)

        assert isinstance(
            received_event, V1BillingMeterErrorReportTriggeredEventNotification
        )
        assert received_event.type == "v1.billing.meter.error_report_triggered"
        assert received_event.id == "evt_123"
        assert received_event.related_object.id == "mtr_123"
        assert isinstance(received_client, StripeClient)

    def test_cannot_register_handler_after_handling(
        self,
        event_handler: StripeEventNotificationHandler,
        v1_billing_meter_payload: str,
    ) -> None:
        """Test that registering handlers after handle() raises RuntimeError"""
        handler = Mock()
        event_handler.on_v1_billing_meter_error_report_triggered(handler)

        sig_header = generate_header(payload=v1_billing_meter_payload)
        event_handler.handle(v1_billing_meter_payload, sig_header)

        with pytest.raises(
            RuntimeError,
            match="Cannot register new event handlers after .handle\\(\\) has been called",
        ):
            event_handler.on_v2_core_account_created(Mock())

    def test_cannot_register_duplicate_handler(
        self, event_handler: StripeEventNotificationHandler
    ) -> None:
        """Test that registering the same event type twice raises ValueError"""
        handler1 = Mock()
        handler2 = Mock()

        event_handler.on_v1_billing_meter_error_report_triggered(handler1)

        with pytest.raises(
            ValueError,
            match='Handler for event type "v1.billing.meter.error_report_triggered" already registered',
        ):
            event_handler.on_v1_billing_meter_error_report_triggered(handler2)

    def test_handler_uses_event_stripe_context(
        self,
        event_handler: StripeEventNotificationHandler,
        v1_billing_meter_payload: str,
        stripe_client: StripeClient,
    ) -> None:
        """Test that the handler receives a client with stripe_context from the event"""
        received_context: Optional[StripeContext | str] = None

        def handler(
            event: V1BillingMeterErrorReportTriggeredEventNotification,
            client: StripeClient,
        ) -> None:
            nonlocal received_context
            received_context = client._requestor._options.stripe_context

        event_handler.on_v1_billing_meter_error_report_triggered(handler)

        assert (
            str(stripe_client._requestor._options.stripe_context)
            == "original_context_123"
        )

        sig_header = generate_header(payload=v1_billing_meter_payload)
        event_handler.handle(v1_billing_meter_payload, sig_header)

        assert str(received_context) == "event_context_456"

    def test_stripe_context_restored_after_handler_success(
        self,
        event_handler: StripeEventNotificationHandler,
        v1_billing_meter_payload: str,
        stripe_client: StripeClient,
    ) -> None:
        """Test that the original stripe_context is restored after successful handler execution"""

        def handler(
            event: V1BillingMeterErrorReportTriggeredEventNotification,
            client: StripeClient,
        ) -> None:
            assert (
                str(client._requestor._options.stripe_context)
                == "event_context_456"
            )

        event_handler.on_v1_billing_meter_error_report_triggered(handler)

        assert (
            str(stripe_client._requestor._options.stripe_context)
            == "original_context_123"
        )

        sig_header = generate_header(payload=v1_billing_meter_payload)
        event_handler.handle(v1_billing_meter_payload, sig_header)

        assert (
            str(stripe_client._requestor._options.stripe_context)
            == "original_context_123"
        )

    def test_stripe_context_restored_after_handler_error(
        self,
        event_handler: StripeEventNotificationHandler,
        v1_billing_meter_payload: str,
        stripe_client: StripeClient,
    ) -> None:
        """Test that the original stripe_context is restored even when handler raises an exception"""

        def handler(
            event: V1BillingMeterErrorReportTriggeredEventNotification,
            client: StripeClient,
        ) -> None:
            assert (
                str(client._requestor._options.stripe_context)
                == "event_context_456"
            )
            raise RuntimeError("Handler error!")

        event_handler.on_v1_billing_meter_error_report_triggered(handler)

        assert (
            str(stripe_client._requestor._options.stripe_context)
            == "original_context_123"
        )

        sig_header = generate_header(payload=v1_billing_meter_payload)

        with pytest.raises(RuntimeError, match="Handler error!"):
            event_handler.handle(v1_billing_meter_payload, sig_header)

        assert (
            str(stripe_client._requestor._options.stripe_context)
            == "original_context_123"
        )

    def test_stripe_context_set_to_none_when_event_has_no_context(
        self,
        event_handler: StripeEventNotificationHandler,
        v2_account_created_payload: str,
        stripe_client: StripeClient,
    ) -> None:
        """Test that stripe_context is set to None when event context is None"""
        received_context: Optional[StripeContext | str] = None

        def handler(
            event: V2CoreAccountCreatedEventNotification, client: StripeClient
        ) -> None:
            nonlocal received_context
            received_context = client._requestor._options.stripe_context

        event_handler.on_v2_core_account_created(handler)

        # Verify we're working with StripeContext instances
        assert isinstance(
            stripe_client._requestor._options.stripe_context, StripeContext
        )
        assert (
            str(stripe_client._requestor._options.stripe_context)
            == "original_context_123"
        )

        sig_header = generate_header(payload=v2_account_created_payload)
        event_handler.handle(v2_account_created_payload, sig_header)

        assert received_context is None

        assert (
            str(stripe_client._requestor._options.stripe_context)
            == "original_context_123"
        )

    def test_unknown_event_routes_to_on_unhandled(
        self,
        event_handler: StripeEventNotificationHandler,
        unknown_event_payload: str,
        fallback_callback: Mock,
    ) -> None:
        """Test that events without SDK types route to on_unhandled handler"""
        sig_header = generate_header(payload=unknown_event_payload)

        event_handler.handle(unknown_event_payload, sig_header)

        fallback_callback.assert_called_once()

        call_args = fallback_callback.call_args[0]
        event_notif = call_args[0]
        client = call_args[1]
        info = call_args[2]

        assert isinstance(event_notif, UnknownEventNotification)
        assert event_notif.type == "llama.created"
        assert isinstance(client, StripeClient)
        assert isinstance(info, UnhandledNotificationDetails)
        assert info.is_known_event_type is False

    def test_known_unregistered_event_routes_to_on_unhandled(
        self,
        event_handler: StripeEventNotificationHandler,
        v1_billing_meter_payload: str,
        fallback_callback: Mock,
    ) -> None:
        """Test that known event types without a registered handler route to on_unhandled"""
        sig_header = generate_header(payload=v1_billing_meter_payload)

        event_handler.handle(v1_billing_meter_payload, sig_header)

        fallback_callback.assert_called_once()

        call_args = fallback_callback.call_args[0]
        event_notif = call_args[0]
        client = call_args[1]
        info = call_args[2]

        assert isinstance(
            event_notif, V1BillingMeterErrorReportTriggeredEventNotification
        )
        assert event_notif.type == "v1.billing.meter.error_report_triggered"
        assert isinstance(client, StripeClient)
        assert isinstance(info, UnhandledNotificationDetails)
        assert info.is_known_event_type is True

    def test_registered_event_does_not_call_on_unhandled(
        self,
        event_handler: StripeEventNotificationHandler,
        v1_billing_meter_payload: str,
        fallback_callback: Mock,
    ) -> None:
        """Test that registered events don't trigger on_unhandled"""
        handler = Mock()
        event_handler.on_v1_billing_meter_error_report_triggered(handler)

        sig_header = generate_header(payload=v1_billing_meter_payload)
        event_handler.handle(v1_billing_meter_payload, sig_header)

        handler.assert_called_once()
        fallback_callback.assert_not_called()

    def test_handler_client_retains_configuration(
        self,
        http_client_mock: HTTPClientMock,
        fallback_callback: Mock,
        v1_billing_meter_payload: str,
    ) -> None:
        """Test that the client passed to handlers retains all configuration except stripe_context"""
        api_key = "sk_test_custom_key"
        original_context = "original_context_xyz"

        client = StripeClient(
            api_key=api_key,
            stripe_context=StripeContext.parse(original_context),
            http_client=http_client_mock.get_mock_http_client(),
        )

        notif_handler = StripeEventNotificationHandler(
            client=client,
            webhook_secret=DUMMY_WEBHOOK_SECRET,
            fallback_callback=fallback_callback,
        )

        received_api_key: Optional[str] = None
        received_context: Optional[StripeContext | str] = None

        def handler(
            event: V1BillingMeterErrorReportTriggeredEventNotification,
            client: StripeClient,
        ) -> None:
            nonlocal received_api_key, received_context
            received_api_key = client._requestor.api_key
            received_context = client._requestor._options.stripe_context

        notif_handler.on_v1_billing_meter_error_report_triggered(handler)

        sig_header = generate_header(payload=v1_billing_meter_payload)
        notif_handler.handle(v1_billing_meter_payload, sig_header)

        assert received_api_key == api_key
        assert str(received_context) == "event_context_456"
        assert (
            str(client._requestor._options.stripe_context) == original_context
        )

    def test_on_unhandled_receives_correct_info_for_unknown(
        self,
        event_handler: StripeEventNotificationHandler,
        unknown_event_payload: str,
        fallback_callback: Mock,
    ) -> None:
        """Test that on_unhandled receives correct UnhandledNotificationDetails for unknown events"""
        sig_header = generate_header(payload=unknown_event_payload)

        event_handler.handle(unknown_event_payload, sig_header)

        fallback_callback.assert_called_once()
        info = fallback_callback.call_args[0][2]

        assert isinstance(info, UnhandledNotificationDetails)
        assert info.is_known_event_type is False

    def test_on_unhandled_receives_correct_info_for_known_unregistered(
        self,
        event_handler: StripeEventNotificationHandler,
        v1_billing_meter_payload: str,
        fallback_callback: Mock,
    ) -> None:
        """Test that on_unhandled receives correct UnhandledNotificationDetails for known unregistered events"""
        sig_header = generate_header(payload=v1_billing_meter_payload)

        event_handler.handle(v1_billing_meter_payload, sig_header)

        fallback_callback.assert_called_once()
        info = fallback_callback.call_args[0][2]

        assert isinstance(info, UnhandledNotificationDetails)
        assert info.is_known_event_type is True

    def test_validates_webhook_signature(
        self,
        event_handler: StripeEventNotificationHandler,
        v1_billing_meter_payload: str,
    ) -> None:
        """Test that invalid webhook signatures are rejected"""
        from stripe._error import SignatureVerificationError

        with pytest.raises(SignatureVerificationError):
            event_handler.handle(v1_billing_meter_payload, "invalid_signature")

    def test_registered_event_types_empty(
        self, event_handler: StripeEventNotificationHandler
    ) -> None:
        """Test that registered_event_types returns empty list when no handlers are registered"""
        assert event_handler.registered_event_types == []

    def test_registered_event_types_single(
        self, event_handler: StripeEventNotificationHandler
    ) -> None:
        """Test that registered_event_types returns a single event type"""
        handler = Mock()
        event_handler.on_v1_billing_meter_error_report_triggered(handler)

        assert event_handler.registered_event_types == [
            "v1.billing.meter.error_report_triggered"
        ]

    def test_registered_event_types_multiple_alphabetized(
        self, event_handler: StripeEventNotificationHandler
    ) -> None:
        """Test that registered_event_types returns multiple event types in alphabetical order"""
        handler = Mock()

        # Register in non-alphabetical order
        event_handler.on_v2_core_account_updated(handler)
        event_handler.on_v1_billing_meter_error_report_triggered(handler)
        event_handler.on_v2_core_account_created(handler)

        expected = [
            "v1.billing.meter.error_report_triggered",
            "v2.core.account.created",
            "v2.core.account.updated",
        ]

        assert event_handler.registered_event_types == expected

    def test_can_call_wrapped_functions(
        self, event_handler: StripeEventNotificationHandler
    ):
        @event_handler.on_v1_billing_meter_error_report_triggered  # type: ignore
        def rand_int(notif, client):
            """cool docstring"""
            return 4

        assert rand_int(None, None) == 4  # type: ignore
